// OScopeCtrl.cpp : implementation file//
/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include "stdafx.h"
#include "math.h"
#include "DS1678.h"

#include "CalenderCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__ ;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCalenderCtrl
CCalenderCtrl::CCalenderCtrl()
{
  // since plotting is based on a LineTo for each new point
  // we need a starting point (i.e. a "previous" point)
  // use 0.0 as the default first point.
  // these are public member variables, and can be changed outside
  // (after construction).  Therefore m_perviousPosition could be set to
  // a more appropriate value prior to the first call to SetPosition.
  m_dPreviousPosition =   0.0 ;

  // public variable for the number of decimal places on the y axis
  m_nYDecimals = 3 ;

  // set some initial values for the scaling until "SetRange" is called.
  // these are protected varaibles and must be set with SetRange
  // in order to ensure that m_dRange is updated accordingly
  m_dLowerLimit = -10.0 ;
  m_dUpperLimit =  10.0 ;
  m_dRange      =  m_dUpperLimit - m_dLowerLimit ;   // protected member variable

  // m_nShiftPixels determines how much the plot shifts (in terms of pixels) 
  // with the addition of a new data point
  m_nShiftPixels     = 4 ;
  m_nHalfShiftPixels = m_nShiftPixels/2 ;                     // protected
  m_nPlotShiftPixels = m_nShiftPixels + m_nHalfShiftPixels ;  // protected

  // background, grid and data colors
  // these are public variables and can be set directly
  m_crBackColor  = RGB(  100, 255,  100) ;  // see also SetBackgroundColor
  m_crGridColor  = RGB(  0, 255, 255) ;  // see also SetGridColor
  m_crPlotColor  = RGB(255, 255, 255) ;  // see also SetPlotColor

  // protected variables
  m_penPlot.CreatePen(PS_SOLID, 0, m_crPlotColor) ;
  m_brushBack.CreateSolidBrush(m_crBackColor) ;


  m_brushEvent.CreateSolidBrush(RGB(  0, 255,   0)) ;
  m_brushOn.CreateSolidBrush(RGB(  255, 0,   0)) ;
  m_brushOff.CreateSolidBrush(RGB(  0, 0,   255)) ;
  m_brushElapsed.CreateSolidBrush(RGB(  0, 255,   0)) ;



  // public member variables, can be set directly 
  m_strXUnitsString.Format("Samples") ;  // can also be set with SetXUnits
  m_strYUnitsString.Format("Y units") ;  // can also be set with SetYUnits

  // protected bitmaps to restore the memory DC's
  m_pbitmapOldGrid = NULL ;
  m_pbitmapOldPlot = NULL ;


	m_CurrentDateTime.wYear = 1980;
	m_CurrentDateTime.wMonth = 1;
	m_CurrentDateTime.wDayOfWeek = 1;
	m_CurrentDateTime.wDay = 1;
	m_CurrentDateTime.wHour = 0;
	m_CurrentDateTime.wMinute = 0;
	m_CurrentDateTime.wSecond = 0;
	m_CurrentDateTime.wMilliseconds = 1;



}  // CCalenderCtrl

/////////////////////////////////////////////////////////////////////////////
CCalenderCtrl::~CCalenderCtrl()
{
  // just to be picky restore the bitmaps for the two memory dc's
  // (these dc's are being destroyed so there shouldn't be any leaks)
  if (m_pbitmapOldGrid != NULL)
    m_dcGrid.SelectObject(m_pbitmapOldGrid) ;  
  if (m_pbitmapOldPlot != NULL)
    m_dcPlot.SelectObject(m_pbitmapOldPlot) ;  

} // ~CCalenderCtrl


BEGIN_MESSAGE_MAP(CCalenderCtrl, CWnd)
  //{{AFX_MSG_MAP(CCalenderCtrl)
  ON_WM_PAINT()
  ON_WM_SIZE()
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CCalenderCtrl message handlers

/////////////////////////////////////////////////////////////////////////////
BOOL CCalenderCtrl::Create(DWORD dwStyle, const RECT& rect, 
                         CWnd* pParentWnd, UINT nID) 
{
  BOOL result ;
  static CString className = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW) ;

  result = CWnd::CreateEx(WS_EX_CLIENTEDGE | WS_EX_STATICEDGE, 
                          className, NULL, dwStyle, 
                          rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
                          pParentWnd->GetSafeHwnd(), (HMENU)nID) ;
  if (result != 0)
    InvalidateCtrl() ;

  return result ;

} // Create

/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::SetRange(double dLower, double dUpper, int nDecimalPlaces)
{
  ASSERT(dUpper > dLower) ;

  m_dLowerLimit     = dLower ;
  m_dUpperLimit     = dUpper ;
  m_nYDecimals      = nDecimalPlaces ;
  m_dRange          = m_dUpperLimit - m_dLowerLimit ;
  m_dVerticalFactor = (double)m_nPlotHeight / m_dRange ; 
  
  // clear out the existing garbage, re-start with a clean plot
  InvalidateCtrl() ;

}  // SetRange


/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::SetXUnits(CString string)
{
  m_strXUnitsString = string ;

  // clear out the existing garbage, re-start with a clean plot
  InvalidateCtrl() ;

}  // SetXUnits

/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::SetYUnits(CString string)
{
  m_strYUnitsString = string ;

  // clear out the existing garbage, re-start with a clean plot
  InvalidateCtrl() ;

}  // SetYUnits

/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::SetGridColor(COLORREF color)
{
  m_crGridColor = color ;

  // clear out the existing garbage, re-start with a clean plot
  InvalidateCtrl() ;

}  // SetGridColor


/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::SetPlotColor(COLORREF color)
{
  m_crPlotColor = color ;

  m_penPlot.DeleteObject() ;
  m_penPlot.CreatePen(PS_SOLID, 0, m_crPlotColor) ;

  // clear out the existing garbage, re-start with a clean plot
  InvalidateCtrl() ;

}  // SetPlotColor


/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::SetBackgroundColor(COLORREF color)
{
  m_crBackColor = color ;

  m_brushBack.DeleteObject() ;
  m_brushBack.CreateSolidBrush(m_crBackColor) ;

  // clear out the existing garbage, re-start with a clean plot
  InvalidateCtrl() ;

}  // SetBackgroundColor

/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::InvalidateCtrl()
{
  // There is a lot of drawing going on here - particularly in terms of 
  // drawing the grid.  Don't panic, this is all being drawn (only once)
  // to a bitmap.  The result is then BitBlt'd to the control whenever needed.
  int i ;
  int nCharacters ;
  int nTopGridPix, nMidGridPix, nBottomGridPix ;

  CPen *oldPen ;
  CPen solidPen(PS_SOLID, 0, m_crGridColor) ;
  CFont axisFont, yUnitFont, *oldFont ;
  CString strTemp ;

  // in case we haven't established the memory dc's
  CClientDC dc(this) ;  

  // if we don't have one yet, set up a memory dc for the grid
  if (m_dcGrid.GetSafeHdc() == NULL)
  {
    m_dcGrid.CreateCompatibleDC(&dc) ;
    m_bitmapGrid.CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight) ;
    m_pbitmapOldGrid = m_dcGrid.SelectObject(&m_bitmapGrid) ;
  }
  
  m_dcGrid.SetBkColor (m_crBackColor) ;

  // fill the grid background
  m_dcGrid.FillRect(m_rectClient, &m_brushBack) ;

  // draw the plot rectangle:
  // determine how wide the y axis scaling values are
  nCharacters = abs((int)log10(fabs(m_dUpperLimit))) ;
  nCharacters = max(nCharacters, abs((int)log10(fabs(m_dLowerLimit)))) ;

  // add the units digit, decimal point and a minus sign, and an extra space
  // as well as the number of decimal places to display
  nCharacters = nCharacters + 4 + m_nYDecimals ;  

  // adjust the plot rectangle dimensions
  // assume 6 pixels per character (this may need to be adjusted)
  m_rectPlot.left = m_rectClient.left + 6*(nCharacters) ;
  m_nPlotWidth    = m_rectPlot.Width() ;

  // draw the plot rectangle
  oldPen = m_dcGrid.SelectObject (&solidPen) ; 
  m_dcGrid.MoveTo (m_rectPlot.left, m_rectPlot.top) ;
  m_dcGrid.LineTo (m_rectPlot.right+1, m_rectPlot.top) ;
  m_dcGrid.LineTo (m_rectPlot.right+1, m_rectPlot.bottom+1) ;
  m_dcGrid.LineTo (m_rectPlot.left, m_rectPlot.bottom+1) ;
  m_dcGrid.LineTo (m_rectPlot.left, m_rectPlot.top) ;
  m_dcGrid.SelectObject (oldPen) ; 

  // draw the dotted lines, 
  // use SetPixel instead of a dotted pen - this allows for a 
  // finer dotted line and a more "technical" look
  nMidGridPix    = (m_rectPlot.top + m_rectPlot.bottom)/2 ;
  nTopGridPix    = nMidGridPix - m_nPlotHeight/4 ;
  nBottomGridPix = nMidGridPix + m_nPlotHeight/4 ;

  for (i=m_rectPlot.left; i<m_rectPlot.right; i+=4)
  {
    m_dcGrid.SetPixel (i, nTopGridPix,    m_crGridColor) ;
    m_dcGrid.SetPixel (i, nMidGridPix,    m_crGridColor) ;
    m_dcGrid.SetPixel (i, nBottomGridPix, m_crGridColor) ;
  }

  // create some fonts (horizontal and vertical)
  // use a height of 14 pixels and 300 weight 
  // (these may need to be adjusted depending on the display)
  axisFont.CreateFont (14, 0, 0, 0, 300,
                       FALSE, FALSE, 0, ANSI_CHARSET,
                       OUT_DEFAULT_PRECIS, 
                       CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY, 
                       DEFAULT_PITCH|FF_SWISS, "Arial") ;
  yUnitFont.CreateFont (14, 0, 900, 0, 300,
                       FALSE, FALSE, 0, ANSI_CHARSET,
                       OUT_DEFAULT_PRECIS, 
                       CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY, 
                       DEFAULT_PITCH|FF_SWISS, "Arial") ;
  
  // grab the horizontal font
  oldFont = m_dcGrid.SelectObject(&axisFont) ;
  
  // y max
  m_dcGrid.SetTextColor (m_crGridColor) ;
  m_dcGrid.SetTextAlign (TA_RIGHT|TA_TOP) ;
  strTemp.Format ("%.*lf", m_nYDecimals, m_dUpperLimit) ;
  m_dcGrid.TextOut (m_rectPlot.left-4, m_rectPlot.top, strTemp) ;

  // y min
  m_dcGrid.SetTextAlign (TA_RIGHT|TA_BASELINE) ;
  strTemp.Format ("%.*lf", m_nYDecimals, m_dLowerLimit) ;
  m_dcGrid.TextOut (m_rectPlot.left-4, m_rectPlot.bottom, strTemp) ;

  // x min
  m_dcGrid.SetTextAlign (TA_LEFT|TA_TOP) ;
  m_dcGrid.TextOut (m_rectPlot.left, m_rectPlot.bottom+4, "0") ;

  // x max
  m_dcGrid.SetTextAlign (TA_RIGHT|TA_TOP) ;
  strTemp.Format ("%d", m_nPlotWidth/m_nShiftPixels) ; 
  m_dcGrid.TextOut (m_rectPlot.right, m_rectPlot.bottom+4, strTemp) ;

  // x units
  m_dcGrid.SetTextAlign (TA_CENTER|TA_TOP) ;
  m_dcGrid.TextOut ((m_rectPlot.left+m_rectPlot.right)/2, 
                    m_rectPlot.bottom+4, m_strXUnitsString) ;

  // restore the font
  m_dcGrid.SelectObject(oldFont) ;

  // y units
  oldFont = m_dcGrid.SelectObject(&yUnitFont) ;
  m_dcGrid.SetTextAlign (TA_CENTER|TA_BASELINE) ;
  m_dcGrid.TextOut ((m_rectClient.left+m_rectPlot.left)/2, 
                    (m_rectPlot.bottom+m_rectPlot.top)/2, m_strYUnitsString) ;
  m_dcGrid.SelectObject(oldFont) ;

  // at this point we are done filling the the grid bitmap, 
  // no more drawing to this bitmap is needed until the setting are changed
  
  // if we don't have one yet, set up a memory dc for the plot
  if (m_dcPlot.GetSafeHdc() == NULL)
  {
    m_dcPlot.CreateCompatibleDC(&dc) ;
    m_bitmapPlot.CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight) ;
    m_pbitmapOldPlot = m_dcPlot.SelectObject(&m_bitmapPlot) ;
  }

  // make sure the plot bitmap is cleared
  m_dcPlot.SetBkColor (m_crBackColor) ;
  m_dcPlot.FillRect(m_rectClient, &m_brushBack) ;

  // finally, force the plot area to redraw
  InvalidateRect(m_rectClient) ;

} // InvalidateCtrl


/////////////////////////////////////////////////////////////////////////////
double CCalenderCtrl::AppendPoint(double dNewPoint)
{
  // append a data point to the plot
  // return the previous point

  double dPrevious ;
  
  dPrevious = m_dCurrentPosition ;
  m_dCurrentPosition = dNewPoint ;
  DrawPoint() ;

  Invalidate() ;

  return dPrevious ;

} // AppendPoint
 

int GetDaysInMonth (int iMonth, int iYear)
{
	static int nMonthLen [] = 
	{	
		31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 
	};

	int nRes = nMonthLen [iMonth - 1];
	if (iMonth == 2 && iYear % 4 == 0 && 
		(iYear % 100 != 0 || iYear % 400 == 0))
	{
		nRes = 29;
	}

	return nRes;
}


CRect			m_rectDayLocations [31];
int				m_iFirstDayOfWeek = 1;
COleDateTime	m_Date;
int				m_iDaysInMonth;

static const int iRows = 6;
static const int iColumns = 7;

void CCalenderCtrl::RebuildLocations (CRect m_rectDays)
{
	//m_Date.SetDate(1999,12,25);
	ReadDate();
	// Get first day of the month:
	COleDateTime dateCurr (m_Date.GetYear (), m_Date.GetMonth (), 1, 0, 0, 0);
	COleDateTimeSpan day (1, 0, 0, 0);

	int y = m_rectDays.top;

	int iRowHeight = m_rectDays.Height () / iRows;
	int iColumnWidth = m_rectDays.Width () / iColumns;

//	while (m_Date.GetMonth () == m_Date.GetMonth ())
	while (dateCurr.GetMonth () == m_Date.GetMonth ())
	{
		int iWeekDay = dateCurr.GetDayOfWeek () - 0;
//		int iWeekDay = 1;
//		int iWeekDay = dateCurr.GetDayOfWeek () - m_iFirstDayOfWeek;
		if (iWeekDay < 1)
		{
			iWeekDay += 7;
		}

		int iDay = dateCurr.GetDay ();
		
		int x = m_rectDays.left + iColumnWidth * (iWeekDay - 1);

		m_rectDayLocations [iDay - 1] = 
			CRect (x, y, x + iColumnWidth, y + iRowHeight);

		if (iWeekDay == 7)
		{
			y += iRowHeight;
			x = m_rectDays.left;
		}
		else
		{
			x += iColumnWidth;
		}

		dateCurr += day;
	}
}


BOOL CCalenderCtrl::GetButtonRect (int iID, CRect& rect)
{
	if (iID < 1 || iID > m_iDaysInMonth)
	{
		return FALSE;
	}

	rect = m_rectDayLocations [iID - 1];
	return TRUE;
}





////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::OnPaint() 
{
/*  CPaintDC dc(this) ;  // device context for painting
  CDC memDC ;
  CBitmap memBitmap ;
  CBitmap* oldBitmap ; // bitmap originally found in CMemDC

  // no real plotting work is performed here, 
  // just putting the existing bitmaps on the client

  // to avoid flicker, establish a memory dc, draw to it 
  // and then BitBlt it to the client
  memDC.CreateCompatibleDC(&dc) ;
  memBitmap.CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight) ;
  oldBitmap = (CBitmap *)memDC.SelectObject(&memBitmap) ;

  if (memDC.GetSafeHdc() != NULL)
  {
    // first drop the grid on the memory dc
    memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
                 &m_dcGrid, 0, 0, SRCCOPY) ;
    // now add the plot on top as a "pattern" via SRCPAINT.
    // works well with dark background and a light plot
    memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
                 &m_dcPlot, 0, 0, SRCPAINT) ;  //SRCPAINT
    // finally send the result to the display
    dc.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
              &memDC, 0, 0, SRCCOPY) ;
  }

  memDC.SelectObject(oldBitmap) ;
*/




	CPaintDC dcPaint (this); // device context for painting

	CRect rectClient;
	GetClientRect (&rectClient);

	//-------------------------------------------------------------------	
	// If possible, we paint to a off-screen DC and BitBlt the results to
	// the screen (reduces flicker).
	//-------------------------------------------------------------------
	CDC*		pDC = &dcPaint;
	BOOL		m_bMemDC = FALSE;
	CDC			dcMem;
	CBitmap		bmp;
	CBitmap*	pOldBmp = NULL;

	if (dcMem.CreateCompatibleDC (&dcPaint) &&
		bmp.CreateCompatibleBitmap (&dcPaint, rectClient.Width (),
								  rectClient.Height ()))
	{
		//-------------------------------------------------------------
		// Off-screen DC successfully created. Better paint to it then!
		//-------------------------------------------------------------
		m_bMemDC = TRUE;
		pOldBmp = dcMem.SelectObject (&bmp);
		pDC = &dcMem;
	}

	pDC->SetBkMode (TRANSPARENT);
	CFont* pOldFont = (CFont*) pDC->SelectStockObject (DEFAULT_GUI_FONT);

	//-----------------
	// Fill background:
	//-----------------
//	RGB (231, 231, 214)
//	pDC->FillSolidRect (&rectClient, m_clrBackColor);
	pDC->FillSolidRect (&rectClient, RGB (231, 231, 214));

	//------------------------------------
	// Draw 3-d border around the control:
	//------------------------------------
//	if (m_bIsDropCalendar)
//	{
		pDC->Draw3dRect (	&rectClient,
						::GetSysColor (COLOR_3DHILIGHT),
						::GetSysColor (COLOR_3DDKSHADOW));
		
		rectClient.InflateRect (-1, -1);
		pDC->Draw3dRect (	&rectClient,
						::GetSysColor (COLOR_3DHILIGHT),
						::GetSysColor (COLOR_3DSHADOW));
/*	}
	else
	{
		pDC->Draw3dRect (	&rectClient, 
						::GetSysColor (COLOR_3DDKSHADOW),
						::GetSysColor (COLOR_3DHILIGHT));
		
		rectClient.InflateRect (-1, -1);
		pDC->Draw3dRect (	&rectClient,
						::GetSysColor (COLOR_3DSHADOW),
						::GetSysColor (COLOR_3DLIGHT));
	}*/

	//----------------------------
	// Get first day of the month:
	//----------------------------
	COleDateTimeSpan day (1, 0, 0, 0);
//	COleDateTime m_Date;
//	m_Date.SetDate(1999,12,25);
	ReadDate();
	COleDateTime dateCurr = m_Date;
	while (dateCurr.GetDay () > 1)
	{
		dateCurr -= day;
	}






	//-----------------------------------------
	// Draw title (calendar's highlighted month):
	//-----------------------------------------
	CRect rectTitle = rectClient;
	rectTitle.bottom = rectTitle.top + rectClient.Height () / 4;

	pDC->FillSolidRect (&rectTitle, ::GetSysColor (COLOR_ACTIVECAPTION));
	pDC->Draw3dRect (	&rectTitle,
					::GetSysColor (COLOR_3DHILIGHT),
					::GetSysColor (COLOR_3DDKSHADOW));

	CString strTitle = m_Date.Format (_T ("%B %Y"));

	
//	AfxMessageBox(strTitle);
//	char str[256];
//	sprintf(str,"%d %d %d",m_Date.GetDay (),
//							m_Date.GetMonth (),
//							m_Date.GetYear ());
//	AfxMessageBox(str);

	pDC->SetTextColor (::GetSysColor (COLOR_CAPTIONTEXT));
	pDC->DrawText (strTitle, rectTitle, DT_SINGLELINE | DT_VCENTER | DT_CENTER);






	//-----------
	// Draw days:
	//-----------
COLORREF		m_clrTextColor;
m_clrTextColor = ::GetSysColor (COLOR_WINDOWTEXT);

  pDC->SetTextColor (m_clrTextColor);

	COleDateTime today = COleDateTime::GetCurrentTime ();

	m_iDaysInMonth = GetDaysInMonth (
						m_Date.GetMonth (),
						m_Date.GetYear ());

	
//	COLORREF		m_clrSelBackColor = ::GetSysColor (COLOR_3DSHADOW);
	COLORREF		m_clrSelBackColor = 	RGB(128, 128, 128);
	for (int iDay = 1; iDay <= m_iDaysInMonth; iDay ++)
	{
		CString strDay;
		strDay.Format (_T ("%d"), iDay);

		BOOL bIsSelectedDay = FALSE; //IsDaySelected (iDay);
		BOOL bIsDayMarked = FALSE; //(m_MarkedDays.Find (iDay) != NULL);

		BOOL bIsToday = (iDay == m_Date.GetDay ()); /* &&
							m_Date.GetMonth () == today.GetMonth () &&
							m_Date.GetYear () == today.GetYear ());*/

		COleDateTime date (m_Date.GetYear (), m_Date.GetMonth (), iDay, 0, 0, 0);
		BOOL bIsIllegalDay = FALSE; //!IsDateValid (date);
		BOOL bHighlight = FALSE;

	    CRect rectDays = rectClient;
	    rectDays.top = rectClient.top + rectClient.Height () / 4;

		RebuildLocations(rectDays);
		CRect rect;
		if (GetButtonRect (iDay, rect)) // && dcPaint.RectVisible (rect))
		{
			if (0) //(bIsSelectedDay)
			{
/*				CRect rectSelect = rect;

				pDC->FillSolidRect (rectSelect, m_clrSelBackColor);
				pDC->SetTextColor (m_clrSelTextColor);

				if (!m_bMultipleSel)
				{
					bHighlight = TRUE;
				}
				else
				{
					CPen penDark (PS_SOLID, 1, m_clrShadowColor);
					CPen* pOldPen = (CPen*) pDC->SelectObject (&penDark);

					if (!IsLeftSelected (iDay))
					{
						pDC->MoveTo (rect.left, rect.top);
						pDC->LineTo (rect.left, rect.bottom);
					}

					if (!IsTopSelected (iDay))
					{
						pDC->MoveTo (rect.left, rect.top);
						pDC->LineTo (rect.right, rect.top);
					}

					CPen penLight (PS_SOLID, 1, m_clrLightColor);
					pDC->SelectObject (&penLight);

					if (!IsRightSelected (iDay))
					{
						pDC->MoveTo (rect.right, rect.top);
						pDC->LineTo (rect.right, rect.bottom);
					}

					if (!IsBottomSelected (iDay))
					{
						pDC->MoveTo (rect.left, rect.bottom - 1);
						pDC->LineTo (rect.right, rect.bottom -  1);
					}

					pDC->SelectObject (pOldPen);
				}*/
			}
			else if (bIsToday)
			{
//				pDC->FillSolidRect (rect, ::GetSysColor (COLOR_INFOBK));
				pDC->FillSolidRect (rect, RGB(150, 255, 150));
				pDC->SetTextColor (::GetSysColor (COLOR_INFOTEXT));

				bHighlight = TRUE;
			}

		/*	if (bIsIllegalDay)
			{
				pDC->SetTextColor (::GetSysColor (COLOR_GRAYTEXT));
			}*/

			
			CFont* pFonfCurr = NULL;
		/*	if (bIsDayMarked)
			{
				pFonfCurr = pDC->SelectObject (&m_fntBold);
			}*/

			pDC->DrawText (strDay, rect, DT_SINGLELINE | DT_VCENTER | DT_CENTER);

			if (pFonfCurr != NULL)
			{
				pDC->SelectObject (pFonfCurr);
			}

		/*	if (bIsSelectedDay || bIsIllegalDay || bIsToday)	// Restore default color
			{
				pDC->SetTextColor (m_clrTextColor);
			}*/

			//---------------------
			// Define button state:
			//---------------------
/*			BUTTON_STATE btnState = FLAT;

			if (iDay == m_iPressedButton)
			{
				if (iDay == m_iTrackedButton)
				{
					btnState = PRESSED;
				}
				else
				{
					btnState = HILIGHT;
				}
			}
			else if (iDay == m_iTrackedButton && m_iPressedButton == 0)
			{
				btnState = HILIGHT;
			}
			else if (bIsToday || bHighlight)
			{
				btnState = PRESSED;
			}

			//------------------------------------
			// Draw 3-d border around of the cell:
			//------------------------------------
			switch (btnState)
			{
			case PRESSED:
				pDC->Draw3dRect (rect, m_clrShadowColor, m_clrLightColor);
				rect.OffsetRect (1, 1);
				break;

			case HILIGHT:
				pDC->Draw3dRect (rect, m_clrLightColor, m_clrShadowColor);
				break;
			}*/
		}
	}

	pDC->SelectObject (pOldFont);

	if (m_bMemDC)
	{
		//--------------------------------------
		// Copy the results to the on-screen DC:
		//-------------------------------------- 
		CRect rect;
		dcPaint.GetClipBox (rect);
		dcPaint.BitBlt (rect.left, rect.top, rect.Width(), rect.Height(),
					   &dcMem, rect.left, rect.top, SRCCOPY);

		dcMem.SelectObject(pOldBmp);
	}








} // OnPaint


#define COLOR_ON       1
#define COLOR_OFF      2
#define COLOR_ELAPSED  3
#define COLOR_EVENT    4
	
struct events_st {
  int color;
  CString str;
} events[12];


/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::DrawPoint()
{
  // this does the work of "scrolling" the plot to the left
  // and appending a new data point all of the plotting is 
  // directed to the memory based bitmap associated with m_dcPlot
  // the will subsequently be BitBlt'd to the client in OnPaint
  CString strTemp;
  int i;

events[0].color = COLOR_EVENT;  events[0].str = "4/9/00 12:12.00";
events[1].color = COLOR_ON;  events[1].str = "4/9/00 12:12.01";
events[2].color = COLOR_EVENT;  events[2].str = "4/9/00 12:12.02";
events[3].color = COLOR_OFF;  events[3].str = "4/9/00 12:12.03";
events[4].color = COLOR_EVENT;  events[4].str = "4/9/00 12:12.04";
events[5].color = COLOR_ON;  events[5].str = "4/9/00 12:12.05";
events[6].color = COLOR_EVENT;  events[6].str = "4/9/00 12:12.06";
events[7].color = COLOR_OFF;  events[7].str = "4/9/00 12:12.07";
events[8].color = COLOR_EVENT;  events[8].str = "4/9/00 12:12.08";
events[9].color = COLOR_ON;  events[9].str = "4/9/00 12:12.09";
events[10].color = COLOR_EVENT; events[10].str = "4/9/00 12:12.10";
events[11].color = COLOR_OFF; events[11].str = "4/9/00 12:12.11";


  CRect rectCleanUp ;
  int flop = 0;

  if (m_dcPlot.GetSafeHdc() != NULL)
  {
    // shift the plot by BitBlt'ing it to itself 
    // note: the m_dcPlot covers the entire client
    //       but we only shift bitmap that is the size 
    //       of the plot rectangle
    // grab the right side of the plot (exluding m_nShiftPixels on the left)
    // move this grabbed bitmap to the left by m_nShiftPixels

/*    m_dcPlot.BitBlt(m_rectPlot.left, m_rectPlot.top+1, 
                    m_nPlotWidth, m_nPlotHeight, &m_dcPlot, 
                    m_rectPlot.left+m_nShiftPixels, m_rectPlot.top+1, 
                    SRCCOPY) ;*/

    // establish a rectangle over the right side of plot
    // which now needs to be cleaned up proir to adding the new point

	  
	  
	  for (i = 0; i < 12; i++) {
		rectCleanUp = m_rectPlot ;
		rectCleanUp.top += (25 * i);
		rectCleanUp.bottom  = rectCleanUp.top + 25;
		// fill the cleanup area with the background
		switch (events[i].color) {
			case COLOR_EVENT:   m_dcPlot.FillRect(rectCleanUp, &m_brushEvent); break;
			case COLOR_ON:      m_dcPlot.FillRect(rectCleanUp, &m_brushOn); break;
			case COLOR_OFF:     m_dcPlot.FillRect(rectCleanUp, &m_brushOff); break;
			case COLOR_ELAPSED: m_dcPlot.FillRect(rectCleanUp, &m_brushElapsed); break;
		}
		rectCleanUp = m_rectPlot ;
		rectCleanUp.top += (25 * i);
		rectCleanUp.bottom  = rectCleanUp.top + 25;
		m_dcPlot.SetTextColor (m_crGridColor) ;
//		m_dcPlot.SetTextAlign (TA_LEFT|TA_TOP) ;
		strTemp.Format ("%s"," 4/9/00 12:2.40") ;
		m_dcPlot.TextOut (rectCleanUp.left+40, rectCleanUp.top+5, events[i].str) ;
	  }




/*    rectCleanUp = m_rectPlot ;
    rectCleanUp.bottom  = rectCleanUp.top + 25 ;

    // fill the cleanup area with the background
    m_dcPlot.FillRect(rectCleanUp, &m_brushOn) ;

    // draw the next line segement
*/
    // grab the plotting pen
/*    oldPen = m_dcPlot.SelectObject(&m_penPlot) ;

    // move to the previous point
    prevX = m_rectPlot.right-m_nPlotShiftPixels ;
    prevY = m_rectPlot.bottom - 
            (long)((m_dPreviousPosition - m_dLowerLimit) * m_dVerticalFactor) ;
    m_dcPlot.MoveTo (prevX, prevY) ;

    // draw to the current point
    currX = m_rectPlot.right-m_nHalfShiftPixels ;
    currY = m_rectPlot.bottom -
            (long)((m_dCurrentPosition - m_dLowerLimit) * m_dVerticalFactor) ;
    m_dcPlot.LineTo (currX, currY) ;

    // restore the pen 
    m_dcPlot.SelectObject(oldPen) ;*/

    // if the data leaks over the upper or lower plot boundaries
    // fill the upper and lower leakage with the background
    // this will facilitate clipping on an as needed basis
    // as opposed to always calling IntersectClipRect
/*    if ((prevY <= m_rectPlot.top) || (currY <= m_rectPlot.top))
      m_dcPlot.FillRect(CRect(prevX, m_rectClient.top, currX+1, m_rectPlot.top+1), &m_brushBack) ;
    if ((prevY >= m_rectPlot.bottom) || (currY >= m_rectPlot.bottom))
      m_dcPlot.FillRect(CRect(prevX, m_rectPlot.bottom+1, currX+1, m_rectClient.bottom+1), &m_brushBack) ;
*/
    // store the current point for connection to the next point
    m_dPreviousPosition = m_dCurrentPosition ;

  }

} // end DrawPoint

/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::OnSize(UINT nType, int cx, int cy) 
{
  CWnd::OnSize(nType, cx, cy) ;

  // NOTE: OnSize automatically gets called during the setup of the control
  
  GetClientRect(m_rectClient) ;

  // set some member variables to avoid multiple function calls
  m_nClientHeight = m_rectClient.Height() ;
  m_nClientWidth  = m_rectClient.Width() ;

  // the "left" coordinate and "width" will be modified in 
  // InvalidateCtrl to be based on the width of the y axis scaling
  m_rectPlot.left   = 20 ;  
  m_rectPlot.top    = 10 ;
  m_rectPlot.right  = m_rectClient.right-10 ;
  m_rectPlot.bottom = m_rectClient.bottom-25 ;

  // set some member variables to avoid multiple function calls
  m_nPlotHeight = m_rectPlot.Height() ;
  m_nPlotWidth  = m_rectPlot.Width() ;

  // set the scaling factor for now, this can be adjusted 
  // in the SetRange functions
  m_dVerticalFactor = (double)m_nPlotHeight / m_dRange ; 

} // OnSize


/////////////////////////////////////////////////////////////////////////////
void CCalenderCtrl::Reset()
{
  // to clear the existing data (in the form of a bitmap)
  // simply invalidate the entire control
  InvalidateCtrl() ;
}



void CCalenderCtrl::SetCurrentDate(SYSTEMTIME m_Time) {

	if ((m_Time.wYear <= 2010) && (m_Time.wYear >= 1980) &&
		(m_Time.wMonth <= 12) && (m_Time.wMonth >= 1) &&
		(m_Time.wDay <= 31) && (m_Time.wDay >= 1)) {
			m_CurrentDateTime = m_Time;
	} else {
		//sprintf(str,"%d %d %d",m_Time.wMonth,m_Time.wDay,m_Time.wYear);
		//AfxMessageBox(str);
		//ASSERT(1);
		m_CurrentDateTime.wYear = 1980;
		m_CurrentDateTime.wMonth = 1;
		m_CurrentDateTime.wDayOfWeek = 1;
		m_CurrentDateTime.wDay = 3;
		m_CurrentDateTime.wHour = 0;
		m_CurrentDateTime.wMinute = 0;
		m_CurrentDateTime.wSecond = 0;
		m_CurrentDateTime.wMilliseconds = 0;
	}
}


void CCalenderCtrl::ReadDate() {
//	SYSTEMTIME m_CurrentDateTime;	// Current LOCAL time.

//	GetDS1678Time(&m_CurrentDateTime);

/*	if ((m_CurrentDateTime.wYear <= 2010) && (m_CurrentDateTime.wYear >= 1980) &&
		(m_CurrentDateTime.wMonth <= 12) && (m_CurrentDateTime.wMonth >= 1) &&
		(m_CurrentDateTime.wDay <= 31) && (m_CurrentDateTime.wDay >= 1)) {*/
	  m_Date.SetDate(m_CurrentDateTime.wYear,
					   m_CurrentDateTime.wMonth,
					   m_CurrentDateTime.wDay);
/*	} else {
		m_Date.SetDate(1980,
				   1,
				   2);
	}*/
}
