/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include "stdafx.h"
#include "sample.h"
#include "PageFiveDialog.h"
#include "DS1678.h"
#include "DS1615io.h"
#include "Serial.h"
#include "CrcErrorDlg.h"
#include "Cmd_Pipe.h"

#include "Cdebug.h"
extern CDebug Debug;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "OutputWindow.h"
extern COutputWindow g_OutputWindow;


/* So that all transactions are syncronized with the timer,
   schedule each user command (Mission Buttons, etc.) in a
   command queue
*/
int cmd_queue_head = 0;
int cmd_queue_tail = 0;


struct cmd_queue_struct {
	int cmd;
	void *parameter1;
	void *parameter2;
	void *parameter3;
	int *status;           // this field is a feedback pointer
	int *page_processed;   // this field tells what page was just processed
	int state;
	DWORD tick;
	int retries;

	int current_page;
	int last_page;
	unsigned char *pbuffer;
} cmd_queue[MAX_CMDS];

void Cmd_Insert(int cmd,void *parameter1,void *parameter2,void *parameter3,int *page_processed,int *status) {
	Debug.Printf3("Cmd_Insert");
	cmd_queue[cmd_queue_head].cmd = cmd;
	cmd_queue[cmd_queue_head].parameter1 = parameter1;
	cmd_queue[cmd_queue_head].parameter2 = parameter2;
	cmd_queue[cmd_queue_head].parameter3 = parameter3;
	cmd_queue[cmd_queue_head].page_processed = page_processed;
	cmd_queue[cmd_queue_head].state = STATE_PREPROCESSING;
	if (status != NULL)
		*status = STATUS_QUEUED;
	cmd_queue[cmd_queue_head].status = status;
	cmd_queue[cmd_queue_head].retries = 0;

	cmd_queue_head++;
	if (cmd_queue_head == MAX_CMDS) {
		cmd_queue_head = 0;
	}
}

bool suspend_timers = false;
bool suspend_due_to_error = false;

// process a single command 
int Cmd_Process() {
	unsigned char ch;
	unsigned char ch2;
	unsigned int  word;
	if (cmd_queue_head != cmd_queue_tail) {
		switch (cmd_queue[cmd_queue_tail].cmd) {
			case CMD_BEGIN_MISSION_SOFTWARE:
					Debug.Printf3("CMD_BEGIN_MISSION_SOFTWARE");
					ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter1);
					DS1615_Begin_Mission(ch);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_BEGIN_MISSION_EVENT:
				break;
			case CMD_END_MISSION:
					Debug.Printf3("CMD_END_MISSION");
					DS1615_End_Mission();
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
					if (cmd_queue[cmd_queue_tail].status) *cmd_queue[cmd_queue_tail].status = STATUS_DONE;
				break;
			case CMD_CLEAR_MEMORY:
					Debug.Printf3("CMD_CLEAR_MEMORY");
					ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter1);
					DS1615_Clear_Memory(ch);
					Sleep(100);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_CLEAR_STENABLE:
					Debug.Printf3("CMD_CLEAR_STENABLE");
					ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter1);
					DS1615_Clear_STEnable(ch);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_CONTROL_REGISTER:
					ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter1);
					Debug.Printf3("CMD_CONTROL_REGISTER %02X",(unsigned int)ch);
					DS1615_Write_Control(ch);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_STATUS_REGISTER:
					ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter1);
					Debug.Printf3("CMD_STATUS_REGISTER %02X",(unsigned int)ch);
					DS1615_Write_Status(ch);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_WRITE_RTC:
					Debug.Printf3("CMD_WRITE_RTC");
					DS1615_Write_RTC((LPSYSTEMTIME)cmd_queue[cmd_queue_tail].parameter1);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_SAMPLE_RATE:
				break;
			case CMD_SAMPLE_DELAY:
					Debug.Printf3("CMD_SAMPLE_DELAY");
					word = *((unsigned int *)cmd_queue[cmd_queue_tail].parameter1);
					DS1615_Start_Delay(word);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_WRITE_REGISTER:
					Debug.Printf3("CMD_WRITE_REGISTER");
					ch  = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter1);
					ch2 = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter2);
					DS1615_Write_Byte(ch,ch2);
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_READ_TEMPERATURE:
					Debug.Printf3("CMD_READ_TEMPERATURE");
					DS1615_Temp_Command();
					cmd_queue[cmd_queue_tail].state = STATE_DONE;
				break;
			case CMD_READ_PAGE:
				g_OutputWindow.PrintLine("CMD_READ_PAGE");
				Debug.Printf3("CMD_READ_PAGE");
				switch (cmd_queue[cmd_queue_tail].state) {
					case STATE_PREPROCESSING:
							g_OutputWindow.PrintLine("STATE_PREPROCESSING");
							Debug.Printf3("STATE_PREPROCESSING");
							ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter2);
							cmd_queue[cmd_queue_tail].current_page = (unsigned int)ch;
							ch = *((unsigned char *)cmd_queue[cmd_queue_tail].parameter3);
							cmd_queue[cmd_queue_tail].last_page    = (unsigned int)ch;
							cmd_queue[cmd_queue_tail].pbuffer = (unsigned char *)(cmd_queue[cmd_queue_tail].parameter1);
							cmd_queue[cmd_queue_tail].state = STATE_QUEUED;
						break;
					case STATE_QUEUED:
							g_OutputWindow.PrintLine("STATE_QUEUED");
							Debug.Printf3("STATE_QUEUED");
							com_in_clear();
							Debug.Printf3("Read_RTC_Page %d",cmd_queue[cmd_queue_tail].current_page);
							Read_Page_CMD(cmd_queue[cmd_queue_tail].current_page);
							cmd_queue[cmd_queue_tail].tick = GetTickCount() + TIMEOUT_PERIOD;
							cmd_queue[cmd_queue_tail].state = STATE_PROCESSING;
						break;
					case STATE_PROCESSING:
							g_OutputWindow.PrintLine("STATE_PROCESSING");
							Debug.Printf3("STATE_PROCESSING");
/*							if ((cmd_queue[cmd_queue_tail].tick < GetTickCount())) {
							    Debug.Printf3("retries");
								cmd_queue[cmd_queue_tail].retries++;
								cmd_queue[cmd_queue_tail].state = STATE_QUEUED;
							}*/
							if (com_in_queued() == 34) {
								g_OutputWindow.PrintLine("com_in_queued() == 34");
							    Debug.Printf3("com_in_queued() == 34");
								if (Read_RTC_Page(cmd_queue[cmd_queue_tail].pbuffer) != -1) {
									// update for status what page was processed
									if (cmd_queue[cmd_queue_tail].page_processed)
										*(cmd_queue[cmd_queue_tail].page_processed) = cmd_queue[cmd_queue_tail].current_page;
									// check if more pages should be read
									if (cmd_queue[cmd_queue_tail].current_page < 
										cmd_queue[cmd_queue_tail].last_page) {
										// advance pointer to end of valid data 
										//   (crc bytes are to be overwritten)
										cmd_queue[cmd_queue_tail].pbuffer += 32;
										// advance to the next page
										cmd_queue[cmd_queue_tail].current_page++;
										// set state machine to requeue Read_Page_CMD
										cmd_queue[cmd_queue_tail].state = STATE_QUEUED;
									} else {
 									// indicate back that command is done
										Debug.Printf3("Read_RTC_Page Done");
										if (cmd_queue[cmd_queue_tail].status) *cmd_queue[cmd_queue_tail].status = STATUS_DONE;
										cmd_queue[cmd_queue_tail].state = STATE_DONE;
									}
								} else {
									Debug.Printf3("bad crc");
									cmd_queue[cmd_queue_tail].retries++;
									cmd_queue[cmd_queue_tail].state = STATE_QUEUED;
								}
						} else {
								unsigned long currenttickcount = GetTickCount();
								unsigned long difference = GetTickCount() - cmd_queue[cmd_queue_tail].tick;
								if ((cmd_queue[cmd_queue_tail].tick < currenttickcount)) {
									Debug.Printf3("retries");
									cmd_queue[cmd_queue_tail].retries++;
									cmd_queue[cmd_queue_tail].state = STATE_QUEUED;
								}
							}
						break;
					case STATE_DONE:

						break;
				}
				break;
		}
	// check if cmd has exceeded retries if so declare inform user of problem
		if (cmd_queue[cmd_queue_tail].retries >= MAX_RETRIES) {
			Debug.Printf3("MAX_RETRIES");
			suspend_due_to_error = true;
//			dialog_open = 1;
			CCrcErrorDlg CrcErrorDlg;
			if (CrcErrorDlg.DoModal() == IDCANCEL) {
				AfxGetMainWnd()->SendMessage(WM_CLOSE); 
//				AfxMessageBox("Shutting Down");
//				queue_shutdown = true;
			} //else {
				cmd_queue[cmd_queue_tail].retries = 0;
//				cmd_queue[cmd_queue_tail].state = STATE_QUEUED;
			//}
			suspend_due_to_error = false;
			// flush the command queue
/*			while (cmd_queue_head != cmd_queue_tail) {
				cmd_queue_tail++;
				if (cmd_queue_tail == MAX_CMDS) {
					cmd_queue_tail = 0;
				}
			}*/
/*			read_timeout = 0;
			dialog_open = 0;
			read_datahist_queued = false;*/
		}
				
	// if this command is done move on to the next command
		if (cmd_queue[cmd_queue_tail].state == STATE_DONE) {
			Debug.Printf3("STATE_DONE");
			cmd_queue_tail++;
			if (cmd_queue_tail == MAX_CMDS) {
				cmd_queue_tail = 0;
			}
		}
	}
	return 0; //CMD_OK;
}







