/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include "stdafx.h"
#include "serial.h"
#include "DS1615io.h"
#include "Sample.h"
#include "CrcErrorDlg.h"
#include "Cdebug.h"
extern CDebug Debug;

int io_mode = MODE_NONE;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int Error_CRC = 0;

int DS1615_Write_Byte(unsigned int address, unsigned char data)
{
	com_in_clear();

	com_putchar3(0x22,(unsigned char)address,data);
	Sleep(10);
	return 1;
}

unsigned int CRC,CRCb,CRCIN;

int calc_crc(unsigned int data) {

  unsigned int i;

  for (i=0; i<8; i++) {
    CRCIN = (((CRC^data)&1) << 15);
    data = data >> 1;
    CRC = CRC >> 1;
    if (CRCIN != 0) {
      CRC = CRC ^ (0xA001);
    }
    CRCb = 0xFFFF - CRC;
  }

  return CRCb;
}

void serial_clear_error();
void set_dialog_open(int i);

extern UINT m_nTimerID2;



//extern bool queue_shutdown;
 



int Read_Page(int page,unsigned char * pbuffer)
{
 int i;
 unsigned char d;
 unsigned int check,c1,c2;
 
//	if (queue_shutdown == true) return -1;

	serial_clear_error();

	Debug.Printf2("Read_Page");
	CRC = 0;
	for (i = 0; i < 32; i ++) {
		d = com_getchar();
		pbuffer[i] = d;
		calc_crc(d);
	}
//	return 1;
	c1 = (int)com_getchar();
	c2 = (int)com_getchar();
	check = c2 * 0x0100 + c1;
	if (check != CRCb) return -1;
	return 1;
}
 

void Read_RTC_CMD() {
  Debug.Printf2("Read_RTC_CMD");
  com_putchar3(0x33,0,0);
}


void Read_Page_CMD(int page) {
 unsigned char a1;
 unsigned char a2;
 unsigned int address;

	Debug.Printf2("Read_Page_CMD");
	address = 0x0020 * page;
	a1 = (unsigned char)(address & 0x00FF);
	a2 = (unsigned char)(address >> 8);

	com_putchar3(0x33,a2,a1);
}


void Read_User_CMD() {
  com_putchar3(0x33,0x00,0x20 * 2);
}


int Read_RTC_Page(unsigned char * pbuffer)
{
	int returned;
	returned = Read_Page(0,pbuffer);
	return returned;
}

int DS1615_Read_Histogram(unsigned int *dat) {
	int i;
	unsigned int val;
	unsigned int valhi;
	unsigned int vallo;
	int index = 0;

	unsigned char buffer[50];

	int j = 0;
	int page;
  int retries = 0;
  DWORD tick;

try_agian:
	Debug.Printf2("Read Histogram Retries = %d",retries);
	for (page = 64; page <= 67; page++)  {
		com_in_clear();
		Read_Page_CMD(page);
		tick = GetTickCount() + 400;
		while (com_in_queued() != 34) {
			if (tick < GetTickCount()) {
				Debug.Printf2("Read Histogram Timeout");
				retries++;
				if (retries == 10) return -1;
				goto try_agian;
			}
		}
		if (Read_Page(page,buffer) == -1) {
			Debug.Printf2("Read Histogram CRC Error");
			retries++;
			if (retries == 10) return -1;
			goto try_agian;
		}
		retries = 0;
	  j = 0;
	  for (i = 0; i < 16; i ++) {
        vallo = (unsigned int)buffer[j++];
        valhi = (unsigned int)buffer[j++];
        val =   (valhi * 0x100) + vallo;
	    dat[index++] = val;
	  }
	}

	Debug.Printf2("Read Histogram Return Ok");
	return 1;
}

int DS1615_Read_Datalog(unsigned int *dat) {
	int i;
	int index = 0;
	unsigned int address = 0x1000;
	unsigned int page;
    unsigned char buffer[50];

  int retries = 0;
  DWORD tick;

	for (page = 128; page <= 191; page++) {
try_agian:
		Debug.Printf2("Read Datalog Retries = %d, Page %d",retries,page);
		com_in_clear();
		Read_Page_CMD(page);
		tick = GetTickCount() + 400;
		while (com_in_queued() != 34) {
			if (tick < GetTickCount()) {
				Debug.Printf2("Read Datalog Timeout");
				retries++;
				if (retries == 10) return -1;
				goto try_agian;
			}
		}
		if (Read_Page(page,buffer) == -1) {
			Debug.Printf2("Read Datalog CRC Error");
			retries++;
			if (retries == 10) return -1;
			goto try_agian;
		}
		retries = 0;
		for (i = 0; i < 32; i ++) 
			dat[index++] = (unsigned int)buffer[i];
    }

	return 1;
}

void DS1615_Clear_Memory(unsigned char old_value) {

	com_in_clear();
	
	// enable a clear through the clear bit
//	com_putchar3(0x22,0x0E,0x40 | old_value);
	com_putchar3(0x22,0x0E,(0x40 | old_value) & 0xEF);
	Sleep(40);
	// issue the clear memory command
	com_putchar(0xA5);

}

void DS1615_Begin_Mission(unsigned char rate) {

	com_in_clear();

	com_putchar3(0x22,0x0D,rate);

}

void DS1615_Clear_STEnable(unsigned char old_value) {

	com_in_clear();

	com_putchar3(0x22,0x0E, old_value & 0xEF);

}


void DS1615_End_Mission() {
  com_in_clear();

  com_putchar3(0x22,0x14,0x00);

}


void DS1615_Start_Delay(unsigned int delay) {
  unsigned char delay_lo,delay_hi;

  delay_lo = (unsigned char)(delay & 0x00FF);
  delay_hi = (unsigned char)(delay >> 8);

  com_in_clear();

  com_putchar3(0x22,0x12,delay_lo);
  Sleep(40);
  com_putchar3(0x22,0x13,delay_hi);


}

unsigned int DS1615_Get_Start_Delay() {

  unsigned char buffer[50];
  unsigned int val,v1,v2;
    
  Read_RTC_Page(buffer);
  v1 = (int)buffer[0x12];	
  v2 = (int)buffer[0x13];
  val = v2 * 0x0100 + v1;
  return val;
}

unsigned int DS1615_Get_Sample_Rate() {

  unsigned char buffer[50];
  unsigned int val;
    
  Read_RTC_Page(buffer);
  val = (int)buffer[0x0D];
  return val;
}

unsigned long DS1615_Get_Current_Sample() {

  unsigned char buffer[50];
  long val,l1,l2,l3;
  DWORD tick;
    
	com_in_clear();
	Read_RTC_CMD();
	tick = GetTickCount() + 500;

	while (com_in_queued() != 34) {
		if (tick < GetTickCount()) {
			Debug.Printf2("DS1615_Get_Current_Sample Timeout");
			return 0;
		}
	}

	if (Read_RTC_Page(buffer) != -1) return 0;
	l1 =  (unsigned long)buffer[0x1A];
	l2 =  (unsigned long)buffer[0x1B];
	l3 =  (unsigned long)buffer[0x1C];
	val = l3 * 0x00010000L + l2 * 0x00000100L + l1;
	return val;
}


void DS1615_Temp_Command() {
    
  com_in_clear();
  com_putchar(0x55);
  Sleep(20);
}





int bcd_to_bin(unsigned char d) {
  int i;
  unsigned char c;

  c = d >> 4;
  i = (int)((c & 0x0F) * 10);
  i += (int)(d & 0x0F);
  return i;
}


unsigned int DS1615BufferToTimeStamp(unsigned char *buffer,LPSYSTEMTIME lpSystemTime) {

// DS1615 memory map for time stamp
//
// 0x15  Minutes
// 0x16  Hours
// 0x17  Date
// 0x18  Month
// 0x19  Year
    lpSystemTime->wSecond		= 0;
	lpSystemTime->wMinute		= bcd_to_bin(buffer[0x15]);
	lpSystemTime->wHour			= bcd_to_bin(buffer[0x16]);
	lpSystemTime->wDayOfWeek	= 1;
	lpSystemTime->wDay			= bcd_to_bin(buffer[0x17]);
	lpSystemTime->wMonth	    = bcd_to_bin(buffer[0x18]);
	lpSystemTime->wYear			= bcd_to_bin(buffer[0x19]) + 2000;

	if (buffer[0x17] == 0)
		return 1;
	else
		return 0;
}

VOID DS1615BufferToCurrentTime(unsigned char *buffer,LPSYSTEMTIME lpSystemTime) {

// DS1615 memory map for current time
//
// 0x01  Minutes
// 0x02  Hours
// 0x04  Date
// 0x05  Month
// 0x06  Year
    lpSystemTime->wSecond		= bcd_to_bin(buffer[0x00]);
	lpSystemTime->wMinute		= bcd_to_bin(buffer[0x01]);
	lpSystemTime->wHour			= bcd_to_bin(buffer[0x02]);
	lpSystemTime->wDayOfWeek	= bcd_to_bin(buffer[0x03]);
	lpSystemTime->wDay			= bcd_to_bin(buffer[0x04]);
	lpSystemTime->wMonth	    = bcd_to_bin(buffer[0x05]);
	lpSystemTime->wYear			= bcd_to_bin(buffer[0x06]) + 2000;

}


unsigned long DS1615BufferToCurrentSample(unsigned char *buffer) {

	long val,l1,l2,l3;

	l1 =  (unsigned long)buffer[0x1A];
	l2 =  (unsigned long)buffer[0x1B];
	l3 =  (unsigned long)buffer[0x1C];
	val = l3 * 0x00010000L + l2 * 0x00000100L + l1;
	return val;
}
/*
VOID SetCurrentDS1678Time() {

SYSTEMTIME SystemTime;

    GetLocalTime(&SystemTime);

	int year,cent;

	year = SystemTime.wYear % 100;
	cent = SystemTime.wYear / 100;

    TWire_Start();
	TWire_Write(0x94);
	TWire_Write(0x00);
	TWire_Write(bin_to_bcd(SystemTime.wSecond));
	TWire_Write(bin_to_bcd(SystemTime.wMinute));
	TWire_Write(bin_to_bcd(SystemTime.wHour));
	TWire_Write(bin_to_bcd(SystemTime.wDayOfWeek));
	TWire_Write(bin_to_bcd(SystemTime.wDay));
	TWire_Write(bin_to_bcd(SystemTime.wMonth));
	TWire_Write(bin_to_bcd(year));
	TWire_Write(bin_to_bcd(cent));
	TWire_Stop();	
}

*/
unsigned char bin_to_bcd(int d) {
  int i;
  unsigned char c;

  i = d / 10;
  c = (unsigned char)(i * 16);
  i = d % 10;
  c += (unsigned char)i;
  return c;
}



void DS1615_Write_RTC(LPSYSTEMTIME lpSystemTime) {
	int year;

	com_in_clear();

	year = lpSystemTime->wYear % 100;

	com_putchar3(0x22,0x00,bin_to_bcd(lpSystemTime->wSecond));
	Sleep(50);
	com_putchar3(0x22,0x01,bin_to_bcd(lpSystemTime->wMinute));
	Sleep(50);
	com_putchar3(0x22,0x02,bin_to_bcd(lpSystemTime->wHour));
	Sleep(50);
	com_putchar3(0x22,0x03,bin_to_bcd(lpSystemTime->wDayOfWeek));
	Sleep(50);
	com_putchar3(0x22,0x04,bin_to_bcd(lpSystemTime->wDay));
	Sleep(50);
	com_putchar3(0x22,0x05,bin_to_bcd(lpSystemTime->wMonth));
	Sleep(50);
	com_putchar3(0x22,0x06,bin_to_bcd(year));

}

VOID DS1615BufferToAlarmTime(unsigned char *buffer,LPSYSTEMTIME lpSystemTime,
						unsigned char *MS,
						unsigned char *MM,
						unsigned char *MH,
						unsigned char *MD
						) {

unsigned char tim[8];

	lpSystemTime->wYear = 1980;
	lpSystemTime->wMonth = 1;
	lpSystemTime->wDayOfWeek = 1;
	lpSystemTime->wDay = 1;
	lpSystemTime->wHour = 0;
	lpSystemTime->wMinute = 0;
	lpSystemTime->wSecond = 0;
	lpSystemTime->wMilliseconds = 1;

	tim[0] = buffer[0x07];
	tim[1] = buffer[0x08];
	tim[2] = buffer[0x09];
	tim[3] = buffer[0x0A];

  // Look at the alarm mask bits and mask them off
	*MS = tim[0] & 0x80;
	tim[0] &= 0x7F;
	*MM = tim[1] & 0x80;
	tim[1] &= 0x7F;
	*MH = tim[2] & 0x80;
	tim[2] &= 0x7F;
	*MD = tim[3] & 0x80;
	tim[3] &= 0x7F;

	if (tim[0] != 127) {
	  lpSystemTime->wSecond = bcd_to_bin(tim[0]);
	  lpSystemTime->wMinute = bcd_to_bin(tim[1]);
	  lpSystemTime->wHour   = bcd_to_bin(tim[2]);
	  lpSystemTime->wDay    = bcd_to_bin(tim[3]);
	}

}


unsigned long DS1615BufferToTotalSamples(unsigned char *buffer) {

	long val,l1,l2,l3;

	l1 =  (unsigned long)buffer[0x1D];
	l2 =  (unsigned long)buffer[0x1E];
	l3 =  (unsigned long)buffer[0x1F];
	val = l3 * 0x00010000L + l2 * 0x00000100L + l1;
	return val;
}

unsigned int DS1615BufferToSampleDelay(unsigned char *buffer) {

  unsigned int val,v1,v2;
    
  v1 = (int)buffer[0x12];	
  v2 = (int)buffer[0x13];
  val = v2 * 0x0100 + v1;
  return val;
}

unsigned int DS1615BufferToSampleRate(unsigned char *buffer) {

  unsigned int val;

  val = (int)buffer[0x0D];
  return val;
}

void DS1615_Write_Control(unsigned char dat) {
  com_in_clear();

  com_putchar3(0x22,0x0E,dat);

}

void DS1615_Write_Status(unsigned char dat) {
  com_in_clear();

  com_putchar3(0x22,0x14,dat);

}



