// OScopeCtrl.cpp : implementation file//
/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include "stdafx.h"
#include "math.h"

#include "Sample.h"
#include "GraphCtrl.h"

#include "Cdebug.h"
extern CDebug Debug;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__ ;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphCtrl
CGraphCtrl::CGraphCtrl()
{
	m_graph_units = TEMPERATURE;
  // since plotting is based on a LineTo for each new point
  // we need a starting point (i.e. a "previous" point)
  // use 0.0 as the default first point.
  // these are public member variables, and can be changed outside
  // (after construction).  Therefore m_perviousPosition could be set to
  // a more appropriate value prior to the first call to SetPosition.
//  m_dPreviousPosition =   0.0 ;

  // public variable for the number of decimal places on the y axis
  m_nYDecimals = 3 ;
  total_x_points = 0;

  // set some initial values for the scaling until "SetRange" is called.
  // these are protected varaibles and must be set with SetRange
  // in order to ensure that m_dRange is updated accordingly
  m_dLowerLimit = -10.0 ;
  m_dUpperLimit =  10.0 ;
  m_dLeftLimit  = 0.0;
  m_dRightLimit = 5.0;
  m_dRange      =  m_dUpperLimit - m_dLowerLimit ;   // protected member variable

  // m_nShiftPixels determines how much the plot shifts (in terms of pixels) 
  // with the addition of a new data point
  m_nShiftPixels     = 4 ;
  m_nHalfShiftPixels = m_nShiftPixels/2 ;                     // protected
  m_nPlotShiftPixels = m_nShiftPixels + m_nHalfShiftPixels ;  // protected

  // background, grid and data colors
  // these are public variables and can be set directly
//  m_crBackColor  = RGB(  0,   0,   0) ;  // see also SetBackgroundColor
//  m_crGridColor  = RGB(  0, 255, 255) ;  // see also SetGridColor
//  m_crPlotColor  = RGB(255, 255, 255) ;  // see also SetPlotColor
  m_crBackColor  = RGB(255, 255, 255) ;  // see also SetBackgroundColor
  m_crGridColor  = RGB(255, 255, 255) ;  // see also SetGridColor
  m_crPlotColor  = RGB(0, 0, 0) ;  // see also SetPlotColor

  // protected variables
  m_penPlot.CreatePen(PS_SOLID, 0, m_crPlotColor) ;
  m_brushBack.CreateSolidBrush(m_crBackColor) ;

  // public member variables, can be set directly 
  m_strXUnitsString.Format("Samples") ;  // can also be set with SetXUnits
  m_strYUnitsString.Format("Y units") ;  // can also be set with SetYUnits

  // protected bitmaps to restore the memory DC's
  m_pbitmapOldGrid = NULL ;
  m_pbitmapOldPlot = NULL ;

  graph_mode = DATALOG;

  IsPointInfo = false;

  last_point.x = 0;
  last_point.y = 0;
  last_point.valid = 0;
//  m_datatip.Create(this);

//  PrintStringNum = 0;

  average_number = 0;

  b_displayticks = false;

  m_XAxisOffset = 0;

  print_graph = true;
}  // CGraphCtrl

/////////////////////////////////////////////////////////////////////////////
CGraphCtrl::~CGraphCtrl()
{
  // just to be picky restore the bitmaps for the two memory dc's
  // (these dc's are being destroyed so there shouldn't be any leaks)
  if (m_pbitmapOldGrid != NULL)
    m_dcGrid.SelectObject(m_pbitmapOldGrid) ;  
  if (m_pbitmapOldPlot != NULL)
    m_dcPlot.SelectObject(m_pbitmapOldPlot) ;  

} // ~CGraphCtrl


BEGIN_MESSAGE_MAP(CGraphCtrl, CWnd)
  //{{AFX_MSG_MAP(CGraphCtrl)
  ON_WM_PAINT()
  ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CGraphCtrl message handlers

/////////////////////////////////////////////////////////////////////////////
BOOL CGraphCtrl::Create(DWORD dwStyle, const RECT& rect, 
                         CWnd* pParentWnd, UINT nID) 
{
  BOOL result ;
  static CString className = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW) ;

  result = CWnd::CreateEx(WS_EX_CLIENTEDGE | WS_EX_STATICEDGE, 
                          className, NULL, dwStyle, 
                          rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
                          pParentWnd->GetSafeHwnd(), (HMENU)nID) ;

  return result ;

} // Create

/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::SetRange(double dLower, double dUpper, int nDecimalPlaces)
{
  ASSERT(dUpper > dLower) ;

  m_dLowerLimit     = dLower ;
  m_dUpperLimit     = dUpper ;
  m_nYDecimals      = nDecimalPlaces ;
  m_dRange          = m_dUpperLimit - m_dLowerLimit ;
  m_dVerticalFactor = (double)m_nPlotHeight / m_dRange ; 
  
  // clear out the existing garbage, re-start with a clean plot
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);
}  // SetRange


/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::SetXUnits(CString string)
{
  m_strXUnitsString = string ;

  // clear out the existing garbage, re-start with a clean plot
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);

}  // SetXUnits

/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::SetYUnits(CString string)
{
  m_strYUnitsString = string ;

  // clear out the existing garbage, re-start with a clean plot
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);

}  // SetYUnits

/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::SetGridColor(COLORREF color)
{
  m_crGridColor = color ;

  // clear out the existing garbage, re-start with a clean plot
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);

}  // SetGridColor


/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::SetPlotColor(COLORREF color)
{
  m_crPlotColor = color ;

  m_penPlot.DeleteObject() ;
  m_penPlot.CreatePen(PS_SOLID, 0, m_crPlotColor) ;

  // clear out the existing garbage, re-start with a clean plot
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);

}  // SetPlotColor


/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::SetBackgroundColor(COLORREF color)
{
  m_crBackColor = color ;

  m_brushBack.DeleteObject() ;
  m_brushBack.CreateSolidBrush(m_crBackColor) ;

  // clear out the existing garbage, re-start with a clean plot
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);

}  // SetBackgroundColor


/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::InvalidateCtrl(CDC *pDC)
{
  last_point.valid = 0;
  int i ;
  int nCharacters ;
  int nTopGridPix, nMidGridPix, nBottomGridPix ;

  CPen *oldPen ;
  CPen solidPen(PS_SOLID, 0, m_crGridColor) ;
  CFont axisFont, yUnitFont, *oldFont ;
  CString strTemp ;

  // in case we haven't established the memory dc's
  CClientDC dc(this) ;  

// set the background color
  pDC->SetBkColor (m_crBackColor) ;
// fill the grid background
  pDC->FillRect(m_rectClient, &m_brushBack) ;

  // draw the plot rectangle:
  // determine how wide the y axis scaling values are
  nCharacters = abs((int)log10(fabs(m_dUpperLimit))) ;
  nCharacters = max(nCharacters, abs((int)log10(fabs(m_dLowerLimit)))) ;

  // add the units digit, decimal point and a minus sign, and an extra space
  // as well as the number of decimal places to display
  nCharacters = nCharacters + 4 + m_nYDecimals ;  

  // adjust the plot rectangle dimensions
  // assume 6 pixels per character (this may need to be adjusted)
  m_rectPlot.left = m_rectClient.left + 6*(nCharacters) + 20 ;
  m_nPlotWidth    = m_rectPlot.Width() ;

// draw the plot rectangle
  oldPen = pDC->SelectObject (&solidPen) ; 
  pDC->MoveTo (m_rectPlot.left, m_rectPlot.top) ;
  pDC->LineTo (m_rectPlot.right+1, m_rectPlot.top) ;
  pDC->LineTo (m_rectPlot.right+1, m_rectPlot.bottom+1) ;
  pDC->LineTo (m_rectPlot.left, m_rectPlot.bottom+1) ;
  pDC->LineTo (m_rectPlot.left, m_rectPlot.top) ;
  pDC->SelectObject (oldPen) ; 

// draw the dotted lines, 
  nMidGridPix    = (m_rectPlot.top + m_rectPlot.bottom)/2 ;
  nTopGridPix    = nMidGridPix - m_nPlotHeight/4 ;
  nBottomGridPix = nMidGridPix + m_nPlotHeight/4 ;

  for (i=m_rectPlot.left; i<m_rectPlot.right; i+=6)
  {
    pDC->SetPixel (i, nTopGridPix,    m_crGridColor) ;
    pDC->SetPixel (i, nMidGridPix,    m_crGridColor) ;
    pDC->SetPixel (i, nBottomGridPix, m_crGridColor) ;
  }

// create font (horizontal and vertical)
  int xpixels_per_inch = pDC->GetDeviceCaps(LOGPIXELSX);
  int ypixels_per_inch = pDC->GetDeviceCaps(LOGPIXELSY);
  int font_height = ypixels_per_inch / 8;	// 1/8 inch font height
  axisFont.CreateFont (font_height, 0, 0, 0, 300,		// set at 0 degrees
                       FALSE, FALSE, 0, ANSI_CHARSET,
                       OUT_DEFAULT_PRECIS, 
                       CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY, 
                       DEFAULT_PITCH|FF_SWISS|TMPF_TRUETYPE , "Arial") ;
  yUnitFont.CreateFont (font_height, 0, 900, 0, 300,	// set at 90 degrees
                       FALSE, FALSE, 0, ANSI_CHARSET,
                       OUT_DEFAULT_PRECIS, 
                       CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY, 
                       DEFAULT_PITCH|FF_SWISS|TMPF_TRUETYPE , "Arial") ;
  
  // grab the horizontal font
  oldFont = pDC->SelectObject(&axisFont) ;


  // draw y units
#define YUNITS 10
	pDC->SetTextColor (m_crGridColor) ;
	pDC->SetTextAlign (TA_RIGHT|TA_TOP) ;
	i = 0;
	for (i = 0; i <= YUNITS; i++) {	
		if (graph_mode == HISTOGRAM) 
			strTemp.Format ("%.*lf", 0, ((((m_dUpperLimit)-m_dLowerLimit)/YUNITS)*i) + m_dLowerLimit);
		else
			strTemp.Format ("%.*lf", m_nYDecimals, ((((m_dUpperLimit)-m_dLowerLimit)/YUNITS)*i) + m_dLowerLimit);
		pDC->TextOut (m_rectPlot.left-(ypixels_per_inch/24), (((m_rectPlot.top-m_rectPlot.bottom)/YUNITS)*i) + m_rectPlot.bottom-(ypixels_per_inch/10)  , strTemp) ;
	}


  // draw x units
#define XUNITS 10
	int xUnits;
	if (graph_mode == DATALOG) {
		pDC->SetTextColor (m_crGridColor);
		pDC->SetTextAlign (TA_RIGHT|TA_TOP);
		if ((m_dRightLimit - m_dLeftLimit) <= 10.0)
			xUnits = (m_dRightLimit - m_dLeftLimit);
		else
			xUnits = XUNITS;
		if (xUnits < 0.9) xUnits = 1.0;
		for (i = 0; i <= xUnits; i++) {	
			strTemp.Format ("%.*lf", 0, ((((m_dRightLimit)-m_dLeftLimit)/xUnits)*i) + m_dLeftLimit + 1.0 + (float)m_XAxisOffset);
			pDC->TextOut ((((m_rectPlot.right-m_rectPlot.left)/xUnits)*i) + m_rectPlot.left + 10, m_rectPlot.bottom+4 , strTemp) ;
		}
	}
	float hist;
	float temp;
	if (graph_mode == HISTOGRAM) {
		if (m_graph_units == TEMPERATURE) {
			hist = -40.0;
			pDC->SetTextColor (m_crGridColor) ;
			pDC->SetTextAlign (TA_RIGHT|TA_TOP) ;
			for (i = 0; i <=  12; i++) {	
				temp = hist + (i * 10.3333);
				temp = (float)ceil(temp);
						if (temp_scale == 'F')
							temp = ((float)1.8 * (float)temp) + (float)32.0;
				strTemp.Format ("%.*lf", m_nYDecimals, temp);
				pDC->TextOut ((((m_rectPlot.right-m_rectPlot.left)/12)*i) + m_rectPlot.left + 10, m_rectPlot.bottom+4 , strTemp) ;
			}
		}
		if (m_graph_units == VOLTAGE) {
			hist = 0.0;
			pDC->SetTextColor (m_crGridColor) ;
			pDC->SetTextAlign (TA_RIGHT|TA_TOP) ;
			for (i = 0; i <=  12; i++) {	
				temp = hist + ((float)i * (2.04 / 12.0));
/*				temp = (float)ceil(temp);
						if (temp_scale == 'F')
							temp = ((float)1.8 * (float)temp) + (float)32.0;*/
				strTemp.Format ("%.*lf", m_nYDecimals, temp);
				pDC->TextOut ((((m_rectPlot.right-m_rectPlot.left)/12)*i) + m_rectPlot.left + 10, m_rectPlot.bottom+4 , strTemp) ;
			}
		}
	}


  // draw x units label
  pDC->SetTextAlign (TA_CENTER|TA_TOP) ;
  pDC->TextOut ((m_rectPlot.left+m_rectPlot.right)/2, 
                    m_rectPlot.bottom+font_height+(ypixels_per_inch/16), m_strXUnitsString) ;

  // restore the font
  pDC->SelectObject(oldFont) ;

  // y units
  oldFont = pDC->SelectObject(&yUnitFont) ;
  pDC->SetTextAlign (TA_CENTER|TA_BASELINE) ;
  pDC->TextOut (m_rectPlot.left-(font_height * 3), 
                    (m_rectPlot.bottom+m_rectPlot.top)/2, m_strYUnitsString) ;
  pDC->SelectObject(oldFont) ;

} // InvalidateCtrl

CRect lrect;
int first = 1;

void CGraphCtrl::DrawRect1(CPoint point1,CPoint point2) {
CDC* ptrDC;

CRect rect (point1.x,point1.y,point2.x,point2.y);

	CPen *oldPen ;
	CPen solidPen(PS_SOLID, 0, m_crGridColor) ;
	CString strTemp ;
	SIZE bsize = { 1, 1};

  // draw the plot rectangle
	ptrDC = GetDC();
	oldPen = ptrDC->SelectObject (&solidPen) ; 
/*  m_dcGrid.MoveTo (m_rectPlot.left, m_rectPlot.top) ;
  m_dcGrid.LineTo (m_rectPlot.right+1, m_rectPlot.top) ;
  m_dcGrid.LineTo (m_rectPlot.right+1, m_rectPlot.bottom+1) ;
  m_dcGrid.LineTo (m_rectPlot.left, m_rectPlot.bottom+1) ;
  m_dcGrid.LineTo (m_rectPlot.left, m_rectPlot.top) ;*/

	if (first) {
		ptrDC->DrawDragRect(rect,bsize,NULL,bsize);
		first = 0;
	} else {
		ptrDC->DrawDragRect(rect,bsize,lrect,bsize);
	}

	lrect = rect;

/*	ptrDC->SetROP2(R2_XORPEN);
	ptrDC->MoveTo(point1.x-4, point1.y-4);
	ptrDC->LineTo(point1.x+5, point1.y+5);
	ptrDC->MoveTo(point1.x-4, point1.y+4);
	ptrDC->LineTo(point1.x+5, point1.y-5);*/
	ptrDC->SelectObject (oldPen) ; 

	ReleaseDC(ptrDC);
}


/////////////////////////////////////////////////////////////////////////////
double CGraphCtrl::AppendPoint(double dNewPoint)
{
  // append a data point to the plot
  // return the previous point

  double dPrevious = 0.0;
  
//  dPrevious = m_dCurrentPosition ;
//  m_dCurrentPosition = dNewPoint ;
  CDC *ptrDC = GetDC();
  DrawPoint(ptrDC) ;
  ReleaseDC(ptrDC);

  Invalidate() ;

  return dPrevious ;

} // AppendPoint

/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::RePaint() //double dNewPoint)
{
  // append a data point to the plot
  // return the previous point

//  double dPrevious ;
  
//  dPrevious = m_dCurrentPosition ;
//  m_dCurrentPosition = dNewPoint ;
  CDC *ptrDC = GetDC();
  DrawPoint(ptrDC) ;
  ReleaseDC(ptrDC);

  Invalidate() ;

//  return dPrevious ;

} // AppendPoint
 
////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::OnPaint() 
{
  CPaintDC dc(this) ;  // device context for painting
  CDC memDC ;
  CBitmap memBitmap ;

  last_point.valid = 0;

  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC);
  DrawPoint(ptrDC) ;
  ReleaseDC(ptrDC);

} // OnPaint



////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::OnPaint2(CDC *pDC) 
{
//  CPaintDC dc(this) ;  // device context for painting
  CDC memDC ;
  CBitmap memBitmap ;
  CBitmap* oldBitmap ; // bitmap originally found in CMemDC

  last_point.valid = 0;

  // no real plotting work is performed here, 
  // just putting the existing bitmaps on the client

  // to avoid flicker, establish a memory dc, draw to it 
  // and then BitBlt it to the client
  memDC.CreateCompatibleDC(pDC) ;
  memBitmap.CreateCompatibleBitmap(pDC, m_nClientWidth, m_nClientHeight) ;
  oldBitmap = (CBitmap *)memDC.SelectObject(&memBitmap) ;

  if (memDC.GetSafeHdc() != NULL)
  {
    // first drop the grid on the memory dc
    memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
                 &m_dcGrid, 0, 0, SRCCOPY) ;
    // now add the plot on top as a "pattern" via SRCPAINT.
    // works well with dark background and a light plot
    memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
//                 &m_dcPlot, 0, 0, SRCCOPY) ;  //SRCPAINT
                 &m_dcPlot, 0, 0, SRCPAINT) ;  //SRCPAINT
    // finally send the result to the display
    pDC->BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
              &memDC, 0, 0, SRCCOPY) ;
  }

  memDC.SelectObject(oldBitmap) ;

} // OnPaint






/*
int total_x_points = 0;
double func[65000];
struct line_struct {
  int x,y;
  double value;
  int index;
} line_info_datalog[3000], line_info_datahist[70],*line_info;
int line_info_points_datalog;
int line_info_points_datahist;
*/
/* = {
	 5, 6, 7, 8, 9,
	10,10,15,15,30,
	30,35,35,-45,-45,
	-55,-55,-45,-45,30,
	30,30,30,30,30,
	31,32,33,34,35
};*/


void CGraphCtrl::AddPoint(double point) {
	func[total_x_points++] = point;
}

void CGraphCtrl::DeletePoints() {
	total_x_points = 0;
}

double xmin_graph;
double xmax_graph;
struct zoom_s {
	double xmin,xmax;
	double xbegin;
	double xend;
} zoom = { 0.0, 0.0, 0.0, 0.0},zoom_rect = { 0.0, 0.0, 0.0, 0.0};

double CGraphCtrl::FindMaxY() {
int i;
double ymax_function;

//find max in function
	ymax_function = 0;
	for (i = (int)zoom.xbegin; i < (int)zoom.xend; i++) {
		if (func[i] > ymax_function) ymax_function = func[i];
	}
	return ymax_function;

}

double CGraphCtrl::FindMinY() {
int i;
double ymin_function;

//find min in function
	ymin_function = 20000;
	for (i = (int)zoom.xbegin; i < (int)zoom.xend; i++) {
		if (func[i] < ymin_function) ymin_function = func[i];
	}
	return ymin_function;

}

double bar_width = 5.0;

bool b_autoscale = true;
/*struct zoom_s {
	double xmin,xmax;
	double xbegin;
	double xend;
} zoom = { 0.0, 0.0, 0.0},zoom_rect = { 0.0, 0.0 , 0.0};*/

void CGraphCtrl::Zoomin() {
	if ((zoom_rect.xmin == 0.0) && (zoom_rect.xmax == 0.0)) {
		MessageBox("You must specify zoom region by clicking mouse and hold while dragging in window","Zoom In",MB_ICONSTOP);
		return;
	}
	b_autoscale = false;
	if ((zoom_rect.xmax + 1.0) < (double)total_x_points) zoom_rect.xmax += 1.0;
	zoom.xmax = zoom_rect.xmax + zoom.xbegin;
	zoom.xmin = zoom_rect.xmin + zoom.xbegin;
	RePaint();
}

void CGraphCtrl::Zoomout() {
	float zoom_out_factor;
	b_autoscale = false;	
	b_zoom_thresholds = false;
	zoom_out_factor = (zoom.xend - zoom.xbegin) * 0.20;
	if ((zoom.xmin - zoom_out_factor) > 0.0) zoom.xmin -= zoom_out_factor; 
		else zoom.xmin = 0.0;

	if ((zoom.xmax + zoom_out_factor) < total_x_points) zoom.xmax += zoom_out_factor; 
		else zoom.xmax = total_x_points;

	RePaint();
}

void CGraphCtrl::Zoomall() {
	b_zoom_thresholds = false;
	b_autoscale = true;	
	RePaint();
}

void CGraphCtrl::Zoomall_Thresholds() {
	b_zoom_thresholds = true;	
	b_autoscale = true;	
	RePaint();
}

void CGraphCtrl::SetHiThreshold(float hi) {
	hi_threshold = hi;
}
void CGraphCtrl::SetLoThreshold(float lo) {
	lo_threshold = lo;
}

void CGraphCtrl::SetXAxisOffset(long offset) {
	m_XAxisOffset = offset;
}

/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::DrawPoint(CDC * pDC)
{

	char str[256];
	sprintf(str,"HORZSIZE %d, VERTSIZE %d",pDC->GetDeviceCaps(HORZSIZE),pDC->GetDeviceCaps(VERTSIZE));
	sprintf(str,"HORZRES %d, VERTRES %d",pDC->GetDeviceCaps(HORZRES),pDC->GetDeviceCaps(VERTRES));
	sprintf(str,"LOGPIXELSX %d, LOGPIXELSY %d",pDC->GetDeviceCaps(LOGPIXELSX),pDC->GetDeviceCaps(LOGPIXELSY));

	int xpixels_per_inch = pDC->GetDeviceCaps(LOGPIXELSX);
	int ypixels_per_inch = pDC->GetDeviceCaps(LOGPIXELSY);
	int font_height = ypixels_per_inch / 8;	// 1/8 inch font height

	int currX, currY ;

	CPen *oldPen ;
	oldPen = pDC->SelectObject(&m_penPlot) ;

	bool b_display_lo_threshold = false;
	bool b_display_hi_threshold = false;


	int i,j;
	int function_length = 30;
	double ypixels;
	double ymin_graph;
	double ymax_graph;
	double yinc_graph;
	double ymin_function;
	double ymax_function;


/*
		// for debug, force all values to the same temp constant
		for (i = 0; i < total_x_points; i++) {
			func[i] = 82.0;
		}
*/



	double xpixels;
	if (b_autoscale) {
		xmin_graph = 0;
		xmax_graph = (double)total_x_points;
	} else {
		xmin_graph = zoom.xmin;
		xmax_graph = zoom.xmax;
	}
	zoom.xbegin = xmin_graph;
	zoom.xend	= xmax_graph;
	//double xmax_graph = (double)150;
	double xinc_graph;

	sprintf(str,"sample 0 %f",func[0]);
	Debug.Printf3(str);
	sprintf(str,"sample 1 %f",func[1]);
	Debug.Printf3(str);
	sprintf(str,"sample 2 %f",func[2]);
	Debug.Printf3(str);
	sprintf(str,"sample 3 %f",func[3]);
	Debug.Printf3(str);

	//find max in function
	ymax_function = 0;
	for (i = xmin_graph; i < xmax_graph; i++) {
		if (func[i] > ymax_function) ymax_function = func[i];
	}

	// sample averaging
	//boolean b_average = true;
	//int average_number = 16; // = 2, 4, 8, 16
	double afunc[8000];
	if ((graph_mode == DATALOG) && (average_number)) {
		for (i = xmin_graph; i < xmax_graph; i++) {
			afunc[i] = func[i];
		}
		for (i = xmin_graph+(average_number/2); i < xmax_graph-(average_number/2); i++) {
			switch (average_number) {
				case 2: 
					afunc[i] =  
								(afunc[i-1] +
								 afunc[i+1]) / 2;
					break;
				case 4: 
					afunc[i] =  
							   (afunc[i-2] +
								afunc[i-1] +
								afunc[i+1] +
								afunc[i+2]) / 4; 
					break;
				case 8: 
					afunc[i] =  
							   (afunc[i-4] + 
								afunc[i-3] +
								afunc[i-2] +
								afunc[i-1] +
								afunc[i+1] +
								afunc[i+2] +
								afunc[i+3] +
								afunc[i+4]) / 8;
					break;
				case 16: 
					afunc[i] =  
							   (afunc[i-8] + 
								afunc[i-7] +
								afunc[i-6] +
								afunc[i-5] +
								afunc[i-4] +
								afunc[i-3] +
								afunc[i-2] +
								afunc[i-1] +
								afunc[i+1] +
								afunc[i+2] +
								afunc[i+3] +
								afunc[i+4] +
								afunc[i+5] +
								afunc[i+6] +
								afunc[i+7] +
								afunc[i+8]) / 16;
					break;
			}
		}
	//	for (i = xmin_graph; i < xmax_graph; i++) {
	//		func[i] = afunc[i];
	//	}
	}


	//find min in function
	ymin_function = 20000;
	for (i = xmin_graph; i < xmax_graph; i++) {
		if (func[i] < ymin_function) ymin_function = func[i];
	}

	//	if the y min and max are the same then adjust the limits to enclose the value
	if (ymin_function == ymax_function) {
		ymin_function -= 1.0;
		ymax_function += 1.0;
	}

	if (b_zoom_thresholds) {
		ymin_function = lo_threshold;
		ymax_function = hi_threshold;
		ymin_function -= 1.0;
		ymax_function += 1.0;
	}
	
	// determine if we should display the hi/lo threshold horz lines
	if (graph_mode == DATALOG) {
		if ((hi_threshold < ymax_function) && (hi_threshold > ymin_function)) {
			b_display_hi_threshold = true;
		}
		if ((lo_threshold < ymax_function) && (lo_threshold > ymin_function)) {
			b_display_lo_threshold = true;
		}
	}
	
	
	if (graph_mode == HISTOGRAM)  // start histogram lower Y limit at zero
		ymin_function = 0;

	/*if (graph_mode == DATALOG) {
		if ((ymax_function - ymax_function) < 20.0) {
			ymin_function+=10;
			ymax_function-=10;
		}
	}
	*/
	OutputDebugString("test");
	// autoscale to min/max of function
	ymin_graph = ymin_function;
	ymax_graph = ymax_function;
	m_dUpperLimit = ymax_graph;
	m_dLowerLimit = ymin_graph;
	m_dLeftLimit  = xmin_graph;
	m_dRightLimit = xmax_graph-1;

	if (total_x_points == 0) {
		m_dUpperLimit = 10;
		m_dLowerLimit = 1;
		m_dLeftLimit  = 0;
		m_dRightLimit = 9;
		Reset();
		return;
	}

	sprintf(str,"xmin_graph %f",xmin_graph);
	Debug.Printf3(str);
	sprintf(str,"xmax_graph %f",xmax_graph);
	Debug.Printf3(str);
	sprintf(str,"ymin_graph %f",ymin_graph);
	Debug.Printf3(str);
	sprintf(str,"ymax_graph %f",ymax_graph);
	Debug.Printf3(str);
	sprintf(str,"m_dUpperLimit %f",m_dUpperLimit);
	Debug.Printf3(str);
	sprintf(str,"m_dLowerLimit %f",m_dLowerLimit);
	Debug.Printf3(str);
	sprintf(str,"m_dLeftLimit %f",m_dLeftLimit);
	Debug.Printf3(str);
	sprintf(str,"m_dRightLimit %f",m_dRightLimit);
	Debug.Printf3(str);
	sprintf(str,"ymax_function %f",ymax_function);
	Debug.Printf3(str);
	sprintf(str,"ymin_function %f",ymin_function);
	Debug.Printf3(str);

	// setup the scale text and grid
	Reset();

	// find number of y pixels in graph
	ypixels = m_rectPlot.bottom - m_rectPlot.top;
	yinc_graph = ypixels / (ymax_graph - ymin_graph);
 
	sprintf(str,"ypixels %f",ypixels);
	Debug.Printf3(str);
	sprintf(str,"yinc_graph %f",yinc_graph);
	Debug.Printf3(str);

	// find number of x pixels in graph
	xpixels = m_rectPlot.right - m_rectPlot.left;
	bar_width = xpixels / 63;
	if ((xmax_graph - xmin_graph - 1) < 1.0)
	  xinc_graph = xpixels;
	else
	  xinc_graph = xpixels / (xmax_graph - xmin_graph - 1);

	sprintf(str,"xpixels %f",xpixels);
	Debug.Printf3(str);
	sprintf(str,"xinc_graph %f",xinc_graph);
	Debug.Printf3(str);

	int xstart = 0;
	int xend   = xmax_graph - xmin_graph;
	if (graph_mode == HISTOGRAM)  // make 63 the last histogram x limit
		xend = 63;
	int xmin = xmin_graph;

//	hi_threshold = 80.0;
//	lo_threshold = 70.0;
	sprintf(str,"hi %f",hi_threshold);
//	AfxMessageBox(str);
	sprintf(str,"lo %f",lo_threshold);
//	AfxMessageBox(str);
	// display dotted threshold lines if needed
	if (graph_mode == DATALOG) {
		if (b_display_hi_threshold) {
			for (i=m_rectPlot.left; i<m_rectPlot.right; i+=2) {
				currY = (int)(m_rectPlot.bottom - ((hi_threshold - ymin_graph) * yinc_graph));
				pDC->SetPixel (i, currY,    RGB(255,0,0));
				pDC->SetPixel (i, currY+1,    RGB(255,0,0));
			}
		}
		if (b_display_lo_threshold) {
			for (i=m_rectPlot.left; i<m_rectPlot.right; i+=2) {
				currY = (int)(m_rectPlot.bottom - ((lo_threshold - ymin_graph) * yinc_graph));
				pDC->SetPixel (i, currY,    RGB(0,0,255));
				pDC->SetPixel (i, currY+1,    RGB(0,0,255));
			}
		}
	}
	bool point_outthere = false;
	// plot function
	if (graph_mode == DATALOG) {
		line_info = line_info_datalog;
		bool first_point = true;
		for (i = xstart; i < xend; i++) {
			currX = (int)(m_rectPlot.left + ((double)i * xinc_graph));
			if (average_number)
				currY = (int)(m_rectPlot.bottom - ((afunc[i+xmin] - ymin_graph) * yinc_graph));
			else
				currY = (int)(m_rectPlot.bottom - ((func[i+xmin] - ymin_graph) * yinc_graph));
			// make sure that the y value is within the plot limits
			if (currY < m_rectPlot.top)		{ currY = m_rectPlot.top; point_outthere = true; }
			if (currY > m_rectPlot.bottom)	{ currY = m_rectPlot.bottom; point_outthere = true; }
			if (first_point)
				pDC->MoveTo (currX, currY);
			else
				pDC->LineTo (currX, currY);
			line_info[i].x = currX;
			line_info[i].y = currY;
			line_info[i].value = func[i+xmin];
			line_info[i].index = i+xmin;
			first_point = false;
/*			if (point_outthere) {
				first_point = true;
				point_outthere = false;
			}*/
		}
		line_info_points_datalog = xend;
		// display the tick marks
		if (b_displayticks) {
			int tick_size = ypixels_per_inch / 30;
			for (i = xstart; i < xend; i++) {
				currX = (int)(m_rectPlot.left + ((double)i * xinc_graph));
				currY = (int)(m_rectPlot.bottom - ((func[i+xmin] - ymin_graph) * yinc_graph));
				// make sure that the y value is within the plot limits
				if (currY < m_rectPlot.top)		currY = m_rectPlot.top;
				if (currY > m_rectPlot.bottom)	currY = m_rectPlot.bottom;
				for (j = -(tick_size/2); j <= (tick_size/2); j++) {
					pDC->MoveTo (currX+j,currY-(tick_size/2));
					pDC->LineTo (currX+j,currY+(tick_size/2));
				}
/*				pDC->MoveTo (currX,currY-tick_size);
				pDC->LineTo (currX,currY+tick_size);
				pDC->MoveTo (currX-1,currY-tick_size);
				pDC->LineTo (currX-1,currY+tick_size);
				pDC->MoveTo (currX+1,currY-tick_size);
				pDC->LineTo (currX+1,currY+tick_size);*/
			}
		}
	}
	if (graph_mode == HISTOGRAM) {
		line_info = line_info_datahist;
		for (i = xstart; i < xend; i++) {
			currX = (int)(m_rectPlot.left + ((double)i * bar_width));
			currY = (int)(m_rectPlot.bottom - ((func[i+xmin] - ymin_graph) * yinc_graph));
			if (func[i+xmin] != 0) {
				pDC->Rectangle(currX,currY,currX + bar_width - 2, m_rectPlot.bottom);
			}
			line_info[i].x = currX;
			line_info[i].y = currY;
			line_info[i].value = func[i+xmin];
			line_info[i].index = i+xmin;
		}
		line_info_points_datahist = xend;
	}

	// plot points
/*	for (i = xmin_graph; i < xmax_graph; i++) {
		currX = (int)(m_rectPlot.left + ((double)i * xinc_graph));
		currY = (int)(m_rectPlot.bottom - ((func[i] - ymin_graph) * yinc_graph));
		m_dcPlot.MoveTo (currX,currY-1);
		m_dcPlot.LineTo (currX,currY+1);
	}*/

    // restore the pen 
    pDC->SelectObject(oldPen) ;


} // end DrawPoint

/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::OnSize(UINT nType, int cx, int cy) 
{
  CWnd::OnSize(nType, cx, cy) ;

  // NOTE: OnSize automatically gets called during the setup of the control
//  CRect rect;
//  GetClientRect(rect) ;

  SizeRectToClient();
  // set some member variables to avoid multiple function calls
/*  m_nClientHeight = m_rectClient.Height() ;
  m_nClientWidth  = m_rectClient.Width() ;
  char str[256];
  sprintf(str,"m_nClientWidth %d  m_nClientHeight %d   ",m_nClientWidth,m_nClientHeight);
  Debug.Printf3(str);

  // the "left" coordinate and "width" will be modified in 
  // InvalidateCtrl to be based on the width of the y axis scaling
  m_rectPlot.left   = 20 ;  
  m_rectPlot.top    = 10 ;
  m_rectPlot.right  = m_rectClient.right-10 ;
  m_rectPlot.bottom = m_rectClient.bottom-45 ;

  // set some member variables to avoid multiple function calls
  m_nPlotHeight = m_rectPlot.Height() ;
  m_nPlotWidth  = m_rectPlot.Width() ;

  // set the scaling factor for now, this can be adjusted 
  // in the SetRange functions
  m_dVerticalFactor = (double)m_nPlotHeight / m_dRange ; 
*/
} // OnSize


void CGraphCtrl::SizeRectToClient() {
  CRect rect;
  GetClientRect(rect) ;

  SizeRect(rect);
}


void CGraphCtrl::SizeRect(CRect rect) {
  
  // set some member variables to avoid multiple function calls

  m_rectClient = rect;

  m_nClientHeight = rect.Height() ;
  m_nClientWidth  = rect.Width() ;
  char str[256];
  sprintf(str,"m_nClientWidth %d  m_nClientHeight %d   ",m_nClientWidth,m_nClientHeight);
  Debug.Printf3(str);

  // the "left" coordinate and "width" will be modified in 
  // InvalidateCtrl to be based on the width of the y axis scaling
  m_rectPlot.left   = 20;  
  m_rectPlot.top    = 10 + rect.top;
  m_rectPlot.right  = rect.right-10 ;
  m_rectPlot.bottom = rect.bottom-45 ;

  // set some member variables to avoid multiple function calls
  m_nPlotHeight = m_rectPlot.Height() ;
  m_nPlotWidth  = m_rectPlot.Width() ;

  // set the scaling factor for now, this can be adjusted 
  // in the SetRange functions
  m_dVerticalFactor = (double)m_nPlotHeight / m_dRange ; 

}


/////////////////////////////////////////////////////////////////////////////
void CGraphCtrl::Reset()
{
  // to clear the existing data (in the form of a bitmap)
  // simply invalidate the entire control
  CDC *ptrDC = GetDC();
  InvalidateCtrl(ptrDC) ;
  ReleaseDC(ptrDC);
}


static bool mouse_down = false;
CRect Selection;
CRect rect_new;
CRect rect_old;
static CPoint point1;
static CPoint point2;


void CGraphCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (graph_mode != DATALOG) return; // prevent zoom-box in histogram mode
	SIZE bsize = { 1, 1};
	OnPaint();

	CRect rect (point1.x,point1.y,point.x,point.y);
	rect_old.SetRectEmpty();
	Debug.Printf3("OnLButtonDown");

	first = 1;
	mouse_down = true;
	point1 = point;
}

CPoint square[4];


void CGraphCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if (graph_mode != DATALOG) return; // prevent zoom-box in histogram mode
	CDC* ptrDC;
	CPen *oldPen ;
	CPen solidPen(PS_SOLID, 0, m_crGridColor) ;

	ptrDC = GetDC();
	oldPen = ptrDC->SelectObject (&solidPen);
	ptrDC->SetROP2(R2_COPYPEN);
	mouse_down = false;
	Debug.Printf3("OnLButtonUp");
	square[0]   = point1;
	square[1].x = point2.x;
	square[1].y = point1.y;
	square[2]   = point2;
	square[3].x = point1.x;
	square[3].y = point2.y;
	ptrDC->MoveTo(square[0]);
	ptrDC->LineTo(square[1]);
	ptrDC->LineTo(square[2]);
	ptrDC->LineTo(square[3]);
	ptrDC->LineTo(square[0]);
	ptrDC->SelectObject (oldPen);
	ReleaseDC(ptrDC);

	Selection.TopLeft() = point1;
	Selection.BottomRight() = point2;

//	TStackedPage::OnLButtonUp(nFlags, point);
}

#define MIN(a,b)	(((a)<(b))?(a):(b))	
#define MAX(a,b)	(((a)>(b))?(a):(b))	

void CGraphCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{

	CDC* ptrDC;
	SIZE bsize = { 1, 1};
	CPen *oldPen ;
	CPen solidPen(PS_SOLID, 0, m_crGridColor) ;
	CString strTemp ;
	bool bswap = false;

  // draw the plot rectangle
	point2 = point;
	if ((point2.x < point1.x) && (point2.y < point1.y)) {
		bswap = true;
	}
	CRect rect;
	rect = CRect(	MIN(point1.x,point2.x),
					MIN(point1.y,point2.y),
					MAX(point1.x,point2.x),
					MAX(point1.y,point2.y));
	if ((mouse_down) && (graph_mode == DATALOG)) {
		ptrDC = GetDC();
		oldPen = ptrDC->SelectObject (&solidPen);
		ptrDC->SetROP2(R2_XORPEN);
		// if there is an old rectangle selection... erase it
		if (!rect_old.IsRectEmpty()) {
				square[0]   = rect_old.TopLeft();
				square[1].x = rect_old.BottomRight().x;
				square[1].y = rect_old.TopLeft().y;
				square[2]   = rect_old.BottomRight();
				square[3].x = rect_old.TopLeft().x;
				square[3].y = rect_old.BottomRight().y;
			ptrDC->MoveTo(square[0]);
			ptrDC->LineTo(square[1]);
			ptrDC->LineTo(square[2]);
			ptrDC->LineTo(square[3]);
			ptrDC->LineTo(square[0]);
			Debug.Printf3("First");
		}
		square[0].x = MIN(point1.x,point2.x);
		square[0].y = MIN(point1.y,point2.y);
		square[1].x = MAX(point1.x,point2.x);
		square[1].y = MIN(point1.y,point2.y);
		square[2].x = MAX(point1.x,point2.x);
		square[2].y = MAX(point1.y,point2.y);
		square[3].x = MIN(point1.x,point2.x);
		square[3].y = MAX(point1.y,point2.y);
		ptrDC->MoveTo(square[0]);
		ptrDC->LineTo(square[1]);
		ptrDC->LineTo(square[2]);
		ptrDC->LineTo(square[3]);
		ptrDC->LineTo(square[0]);
		ptrDC->SelectObject (oldPen);
		Debug.Printf3("Second");
		ReleaseDC(ptrDC);
		rect_old = rect;
		// now find zoom indices
		if (graph_mode == DATALOG) {
			int i;
			double swaptemp;
			bool bpoint_in_graph;
			// get the first zoom point...
			for (i = 0; i < (line_info_points_datalog - 0); i++) {
				// check if mouse in region of plot point
				// .. first check if point is between plot points
				if (i < (line_info_points_datalog - 1))
					bpoint_in_graph = ((line_info[i].x <= point1.x) && (line_info[i+1].x >= point1.x));
				else  // else check if the point is on right-most of graph
					bpoint_in_graph = (line_info[i].x <= point1.x);
				if (bpoint_in_graph) {
					zoom_rect.xmin = (double)i;
				}
			}
			// check if the point is on the left-most of the graph
			if (line_info[line_info_points_datalog-1].x <= point1.x) 
				zoom_rect.xmax = (double)(line_info_points_datalog-1);
			// check if the point is on the left-most of the graph
			if (line_info[0].x >= point1.x) 
				zoom_rect.xmin = (double)(0.0);
			// get the second zoom point...
			for (i = 0; i < (line_info_points_datalog - 0); i++) {
				// check if mouse in region of plot point
				// .. first check if point is between plot points
				if (i < (line_info_points_datalog - 1))
					bpoint_in_graph = ((line_info[i].x <= point2.x) && (line_info[i+1].x >= point2.x));
//				else  // else check if the point is on right-most of graph
//					bpoint_in_graph = (line_info[i].x <= point2.x);
				if (bpoint_in_graph) {
					zoom_rect.xmax = (double)i;
				}
			}
			// check if the point is on the right-most of the graph
			if (line_info[line_info_points_datalog-1].x <= point2.x) 
				zoom_rect.xmax = (double)(line_info_points_datalog-1);
			// check if the point is on the left-most of the graph
			if (line_info[0].x >= point2.x) 
				zoom_rect.xmax = (double)(0.0);
			// swap the min and max points if needed
			if (zoom_rect.xmin > zoom_rect.xmax) {
				swaptemp = zoom_rect.xmin;
				zoom_rect.xmin = zoom_rect.xmax;
				zoom_rect.xmax = swaptemp;
			}
		}
	} else {
		int i;
		char str[128];
		bool bpoint_in_graph;
		int line_info_points;
		// determine which point set to look at (datalog or histogram)
		if (graph_mode == DATALOG) 
			line_info_points = line_info_points_datalog;
		else
			line_info_points = line_info_points_datahist;
		if (graph_mode == DATALOG) 
			line_info = line_info_datalog;
		else
			line_info = line_info_datahist;
		if (line_info_points >= 2) {
			for (i = 0; i < (line_info_points - 0); i++) {
				// check if mouse in region of plot point
				// .. first check if point is between plot points
				if (i < (line_info_points - 1))
					bpoint_in_graph = ((line_info[i].x <= point.x) && (line_info[i+1].x >= point.x));
				else  // else check if the point is on right-most of graph
					bpoint_in_graph = (line_info[i].x <= point.x);
				if (bpoint_in_graph) {
				  // hilight point
					CDC* ptrDC;
					CPen *oldPen ;
					CPen solidPen(PS_SOLID, 0, RGB(  255, 255, 0)) ;
					CPen solidPen2(PS_SOLID, 0, RGB(  255, 255, 255)) ;
					ptrDC = GetDC();
					oldPen = ptrDC->SelectObject (&solidPen);
					ptrDC->SetROP2(R2_XORPEN);
					int currX = line_info[i].x;
					int currY = line_info[i].y;

					ptrDC->MoveTo (currX,m_rectPlot.top);
					ptrDC->LineTo (currX,m_rectPlot.bottom);
					if (graph_mode == HISTOGRAM) {
						for (int w = 1; w < (int)bar_width-2;w++) {
							ptrDC->MoveTo (currX+w,m_rectPlot.top);
							ptrDC->LineTo (currX+w,m_rectPlot.bottom);
						}
					}
					//ptrDC->MoveTo (currX-1,currY-2);
					//ptrDC->LineTo (currX-1,currY+2);
					//ptrDC->MoveTo (currX+1,currY-2);
					//ptrDC->LineTo (currX+1,currY+2);
					if (last_point.valid) {
						//ptrDC->SelectObject (&solidPen2);
						currX = last_point.x;
						currY = last_point.y;
						ptrDC->MoveTo (currX,m_rectPlot.top);
						ptrDC->LineTo (currX,m_rectPlot.bottom);
						if (graph_mode == HISTOGRAM) {
							for (int w = 1; w < (int)bar_width-2;w++) {
								ptrDC->MoveTo (currX+w,m_rectPlot.top);
								ptrDC->LineTo (currX+w,m_rectPlot.bottom);
							}
						}
/*						ptrDC->MoveTo (currX,currY-2);
						ptrDC->LineTo (currX,currY+2);
						ptrDC->MoveTo (currX-1,currY-2);
						ptrDC->LineTo (currX-1,currY+2);
						ptrDC->MoveTo (currX+1,currY-2);
						ptrDC->LineTo (currX+1,currY+2);*/
					}
					last_point.valid = 1;
					last_point.x = line_info[i].x;
					last_point.y = line_info[i].y;

					ptrDC->SelectObject (oldPen);
					ReleaseDC(ptrDC);

					//m_datatip.Set(point, "message");

					sprintf(str,"%3.1f",line_info[i].value);

					// tell the calling dialog that there is cursor info
					//   give it the value at cursor, the index at cursor
					IsPointInfo = true;
					value = line_info[i].value;
					index = line_info[i].index;
					mousex = point.x;
					mousey = point.y;
					cursorx = line_info[i].x;
					cursory = line_info[i].y;
					index_into_line_info = i;

					//ASSERT(GetDlgItem(IDC_TEXT_TEMPERATURE));
					//GetDlgItem(IDC_TEXT_TEMPERATURE)->SetWindowText(str);

				  // display point index and value
					//sprintf(str,"%d %f",line_info[i].index,line_info[i].value);
					//Debug.Printf3(str);
				}
			}
		}
	}
//	TStackedPage::OnMouseMove(nFlags, point);
}

void CGraphCtrl::SetHistogram() {
  graph_mode = HISTOGRAM;
}
void CGraphCtrl::SetDataLog() {
  graph_mode = DATALOG;
}
void CGraphCtrl::SetC() {
  temp_scale = 'C';
}
void CGraphCtrl::SetF() {
  temp_scale = 'F';
}




void CGraphCtrl::SetFilter(int f) {
	average_number = f;
	Invalidate() ;
}

void CGraphCtrl::DisplayTicks(boolean b) {
	b_displayticks = b;
	Invalidate() ;
}

s_printstrings *lprinthead;
bool printing_done = false;
int currenty = 0;

void CGraphCtrl::PrintGraph() {

			Debug.Printf0("PrintGraph()");
			Sleep(500);
	printing_done = false;
	OnFilePrint2();

}


void CGraphCtrl::OnPrint(CDC * pDC, CPrintInfo * pInfo)
{
//OnPaint2(pDC);
			Debug.Printf0("OnPrint()");
			Sleep(500);

	char str[256];
	sprintf(str,"CPrintInfo->m_rectDraw %d %d %d %d",
	pInfo->m_rectDraw.left,
	pInfo->m_rectDraw.top,
	pInfo->m_rectDraw.right,
	pInfo->m_rectDraw.bottom);
//	AfxMessageBox(str);

//	sprintf(str,"HORZSIZE %d, VERTSIZE %d",pDC->GetDeviceCaps(HORZSIZE),pDC->GetDeviceCaps(VERTSIZE));
//	AfxMessageBox(str);
	sprintf(str,"HORZRES %d, VERTRES %d",pDC->GetDeviceCaps(HORZRES),pDC->GetDeviceCaps(VERTRES));
//	AfxMessageBox(str);
//	sprintf(str,"LOGPIXELSX %d, LOGPIXELSY %d",pDC->GetDeviceCaps(LOGPIXELSX),pDC->GetDeviceCaps(LOGPIXELSY));
//	AfxMessageBox(str);
	sprintf(str,"pInfo->m_nCurPage %d",pInfo->m_nCurPage);
//	AfxMessageBox(str);


	// figure out logical units per mm 
	float xunits_per_mm;
	xunits_per_mm = pInfo->m_rectDraw.right / 215;	// I am calling a standard sheet of 
	float yunits_per_mm;							// 8.5x11 paper  215mm x 280mm
	yunits_per_mm = pInfo->m_rectDraw.bottom / 280;

	CRect rect = pInfo->m_rectDraw;
	rect.bottom = rect.bottom / 2;

	// bring left and right edges in 1cm
	rect.left += xunits_per_mm * 10;
	rect.right -= xunits_per_mm * 10;

	// shift plot down 1cm
	rect.top += yunits_per_mm * 10;
	rect.bottom += yunits_per_mm * 10;

	
			Debug.Printf0("save plot colors");
			Sleep(500);
	// save plot colors
	COLORREF m_crBackColor_save = m_crBackColor;
	COLORREF m_crGridColor_save = m_crGridColor;
	COLORREF m_crPlotColor_save = m_crPlotColor;
	// set plot colors to back and white
	SetBackgroundColor(RGB(255, 255, 255)) ;
	SetGridColor(RGB(0, 0, 0));
	SetPlotColor(RGB(0, 0, 0));

	// set plot rectangle with margins
			Debug.Printf0("SizeRect()");
			Sleep(500);
	SizeRect(rect);

//	if (!tabulated_printout) {
	if (print_graph) {
		// draw plot and labels
			Debug.Printf0("InvalidateCtrl()");
			Sleep(500);
		InvalidateCtrl(pDC);
			Debug.Printf0("DrawPoint()");
			Sleep(500);
		DrawPoint(pDC) ;
	// draw the cursor
		if (IsPointInfo) {
			Debug.Printf0("IsPointInfo");
			Sleep(500);
			CPen *oldPen ;
			CPen solidPen(PS_SOLID, 0, RGB(  0, 0, 0)) ;
			//CPen solidPen2(PS_SOLID, 0, RGB(  255, 255, 255)) ;
//				ptrDC = GetDC();
			oldPen = pDC->SelectObject (&solidPen);
//			pDC->SetROP2(R2_XORPEN);
			if (graph_mode == DATALOG) 
				line_info = line_info_datalog;
			else
				line_info = line_info_datahist;
			int currX = line_info[index_into_line_info].x;
			int currY = line_info[index_into_line_info].y;

			int l;
			if (graph_mode == DATALOG) {
				for (l = m_rectPlot.top; l < m_rectPlot.bottom; l+=6) {
					pDC->MoveTo (currX,l);
					pDC->LineTo (currX,l+3);
				}
			}
			if (graph_mode == HISTOGRAM) {
				for (l = m_rectPlot.top; l < m_rectPlot.bottom; l+=6) {
					pDC->MoveTo (currX+bar_width/2,l);
					pDC->LineTo (currX+bar_width/2,l+3);
				}
/*				for (int w = 1; w < (int)bar_width-2;w++) {
					pDC->MoveTo (currX+w,m_rectPlot.top);
					pDC->LineTo (currX+w,m_rectPlot.bottom);
				}*/
			}
			pDC->SelectObject (oldPen);
		}
	}

	// draw text after plot
	CFont axisFont, yUnitFont, *oldFont ;
	CString strTemp ;
	int xpixels_per_inch = pDC->GetDeviceCaps(LOGPIXELSX);
	int ypixels_per_inch = pDC->GetDeviceCaps(LOGPIXELSY);
	int font_height = ypixels_per_inch / 7;	// 1/8 inch font height

// create font (horizontal and vertical)
			Debug.Printf0("CreateFont()");
			Sleep(500);
	axisFont.CreateFont (font_height, 0, 0, 0, 300,		// set at 0 degrees
					   FALSE, FALSE, 0, ANSI_CHARSET,
					   OUT_DEFAULT_PRECIS, 
					   CLIP_DEFAULT_PRECIS,
					   DEFAULT_QUALITY, 
					   DEFAULT_PITCH|FF_SWISS|TMPF_TRUETYPE , "Arial") ;
	oldFont = pDC->SelectObject(&axisFont) ;
	pDC->SetTextColor (m_crGridColor) ;
	pDC->SetTextAlign (TA_LEFT|TA_TOP) ;

	int y = 0;
	if (print_graph) {
		currenty = rect.bottom;
	} else {
		currenty = font_height*2;
	}
	// only print the graph on the first page
	print_graph = false;
	// display title of plot at top of plot on all pages
			Debug.Printf0("TextOut(), title");
			Sleep(500);
	pDC->TextOut (pDC->GetDeviceCaps(HORZRES)/2,font_height , TitleStr);

	// if tabulated, display that first
	s_printstrings *ptemp;
	// if this is the first page then point to the head of the string list
	if (pInfo->m_nCurPage == 1) {
		lprinthead = lhead;
	}
	while (lprinthead) {
		pDC->TextOut (rect.left, currenty , *lprinthead->cstr);
		currenty+=font_height;
		// delete CString
		delete lprinthead->cstr;
		ptemp = lprinthead->next;
		// delete the CString structure
		delete lprinthead;
		lprinthead = ptemp;
		// check if we are getting to the end of the page
		//  ... if so, end this page and make the framework loop around to start another one
		if (currenty > (pDC->GetDeviceCaps(VERTRES) - (font_height * 5))) {
//			AfxMessageBox("more");
			// because there is more that one page... display the page number
			sprintf(str,"Page %d",pInfo->m_nCurPage);
			pDC->TextOut (pDC->GetDeviceCaps(HORZRES)/2,pDC->GetDeviceCaps(VERTRES) - font_height - font_height/2 , str);
			goto exit_print;	// we have more info than what can fit on a page
		}
	}

	// this is the last page to print
	printing_done = true;
//	AfxMessageBox("done");
	// if there are multiple pages then display the page number of the last page
	if (pInfo->m_nCurPage != 1) {
		sprintf(str,"Page %d",pInfo->m_nCurPage);
		pDC->TextOut (pDC->GetDeviceCaps(HORZRES)/2,pDC->GetDeviceCaps(VERTRES) - font_height - font_height/2 , str);
	}
	goto all_done;

exit_print:
	printing_done = false;
all_done:
	// restore font
			Debug.Printf0("SelectObject(oldFont)");
			Sleep(500);
	pDC->SelectObject(oldFont) ;
///////////


	// restore to screen dimensions
			Debug.Printf0("SizeRectToClient()");
			Sleep(500);
	SizeRectToClient();
	// restore plot colors
			Debug.Printf0("SetBackgroundColor()");
			Sleep(500);
	SetBackgroundColor(m_crBackColor_save) ;
	SetGridColor(m_crGridColor_save) ;
	SetPlotColor(m_crPlotColor_save) ;
}


void CGraphCtrl::OnPrepareDC(CDC * pDC, CPrintInfo * pInfo)
{
//	AfxMessageBox("OnPrepareDC");
			Debug.Printf0("OnPrepareDC()");
			Sleep(500);
	if (printing_done == true) {
		pInfo->m_bContinuePrinting = false;
//		AfxMessageBox("false");
	}
	else {
		pInfo->m_bContinuePrinting = true;
//		AfxMessageBox("true");
	}
}
