// OScopeCtrl.h : header file
//
/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#ifndef __CGraphCtrl_H__
#define __CGraphCtrl_H__

#define HISTOGRAM 1
#define DATALOG   2

#include "PrinterJob.h"



struct s_printstrings {
  CString *cstr;
  s_printstrings *next;
};

#define TEMPERATURE 0
#define VOLTAGE     1

/////////////////////////////////////////////////////////////////////////////
// CGraphCtrl window

class CGraphCtrl : public CPrinterJob
{
// Construction
public:
  CGraphCtrl();

int m_graph_units;

int total_x_points;
double func[65000];
struct line_struct {
  int x,y;
  double value;
  int index;
} line_info_datalog[3000], line_info_datahist[70],*line_info;
int index_into_line_info;
int line_info_points_datalog;
int line_info_points_datahist;
int average_number; // = 0, 2, 4, 8, 16
boolean b_displayticks;

// Attributes
public:
  void SetGraphUnits(int unit) { m_graph_units = unit; }
  int  GetGraphUnits() { return m_graph_units; }
  double AppendPoint(double dNewPoint);
  void SetRange(double dLower, double dUpper, int nDecimalPlaces=1);
  void SetXUnits(CString string);
  void SetYUnits(CString string);
  void SetGridColor(COLORREF color);
  void SetPlotColor(COLORREF color);
  void SetBackgroundColor(COLORREF color);
  void InvalidateCtrl(CDC *pDC);
  void DrawPoint(CDC * pDC);
  void Reset();
  void AddPoint(double point);
  void DeletePoints();
  void SetHistogram();
  void SetDataLog();
  void SetC();
  void SetF();
  double FindMaxY();
  double FindMinY();
  void RePaint();
  void PrintGraph();


// override functions
void OnPrint(CDC* pDC, CPrintInfo* pInfo);
void OnPrepareDC(CDC * pDC, CPrintInfo * pInfo);

//TFXDataTip m_datatip;
void OnPaint2(CDC *pDC);
void SizeRect(CRect rect);
void SizeRectToClient();

void SetFilter(int f);
void DisplayTicks(boolean b);


void Zoomin();
void Zoomout();
void Zoomall();

void SetHiThreshold(float hi);
void SetLoThreshold(float lo);

long m_XAxisOffset;
void SetXAxisOffset(long offset);

void Zoomall_Thresholds();

bool b_zoom_thresholds;

  // this data feedbacks info about mouse location in the graph
  float hi_threshold;
  float lo_threshold;

  bool IsPointInfo;
  int mousex;
  int mousey;
  int index;
  double value;
  int cursorx;
  int cursory;

  bool print_graph;
  CString TitleStr;

//  CString PrintString[128];
  s_printstrings *lPrintStrings,*lhead;
//  int     PrintStringNum;

struct last_struct {
	int x,y;
	int valid;
} last_point;


// selection fuctions
void DrawRect1(CPoint point1,CPoint point2);

  // Operations
public:

// Overrides
  // ClassWizard generated virtual function overrides
  //{{AFX_VIRTUAL(CGraphCtrl)
  public:
  virtual BOOL Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID=NULL);
  //}}AFX_VIRTUAL

// Implementation
public:
  int m_nShiftPixels;          // amount to shift with each new point 
  int m_nYDecimals;

  CString m_strXUnitsString;
  CString m_strYUnitsString;

  COLORREF m_crBackColor;        // background color
  COLORREF m_crGridColor;        // grid color
  COLORREF m_crPlotColor;        // data color  
  
  double m_dCurrentPosition;   // current position
  double m_dPreviousPosition;  // previous position

  virtual ~CGraphCtrl();

  // Generated message map functions
protected:
  //{{AFX_MSG(CGraphCtrl)
  afx_msg void OnPaint();
  afx_msg void OnSize(UINT nType, int cx, int cy); 
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
  //}}AFX_MSG
  DECLARE_MESSAGE_MAP()

  int m_nHalfShiftPixels;
  int m_nPlotShiftPixels;
  int m_nClientHeight;
  int m_nClientWidth;
  int m_nPlotHeight;
  int m_nPlotWidth;

  double m_dLowerLimit;        // lower bounds
  double m_dUpperLimit;        // upper bounds
  double m_dLeftLimit;         // lower bounds
  double m_dRightLimit;        // upper bounds
  double m_dRange;
  double m_dVerticalFactor;
  
  CRect  m_rectClient;
  CRect  m_rectPlot;
  CPen   m_penPlot;
  CBrush m_brushBack;

  CDC     m_dcGrid;
  CDC     m_dcPlot;
  CBitmap *m_pbitmapOldGrid;
  CBitmap *m_pbitmapOldPlot;
  CBitmap m_bitmapGrid;
  CBitmap m_bitmapPlot;

  int graph_mode;
  char temp_scale;

};

/////////////////////////////////////////////////////////////////////////////
#endif
