// OutputWindow.cpp: implementation of the COutputWindow class.
/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include "stdafx.h"
#include "OutputWindow.h"

// Init static member variables
HWND COutputWindow::m_hWnd = 0;
HWND COutputWindow::m_hWndEdit = 0;
unsigned int COutputWindow::m_iInstCount = 0;

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

COutputWindow::COutputWindow()
{
	// One more instance of this class
	m_iInstCount++;
}

COutputWindow::~COutputWindow()
{
	// Clean up

	// Decrement instance count
	m_iInstCount--;

	// Last instance of the class destroyed ?
	if (m_iInstCount == 0)
	{
		// Destroy window
		if (m_hWnd)
			DestroyWindow(m_hWnd);
	}
}

bool COutputWindow::CreateOutputWindow(HINSTANCE hInst, HWND hWndParent, LPRECT prcWindowPosition)
{
	// Create the output window

	WNDCLASSEX wcex;

	// Don't create the window twice
	if (m_hWnd)
		return FALSE;

	// Register the window class
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= NULL;
	wcex.lpfnWndProc	= (WNDPROC) WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInst;
	wcex.hIcon			= LoadIcon(NULL, (LPCTSTR) IDI_WINLOGO);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH) GetStockObject(GRAY_BRUSH);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= OUTPUT_WINDOW_CLASS;
	wcex.hIconSm		= LoadIcon(NULL, (LPCTSTR) IDI_WINLOGO);

	if (!RegisterClassEx(&wcex))
		return FALSE;

	// Create the window
	m_hWnd = CreateWindowEx(WS_EX_TOOLWINDOW, OUTPUT_WINDOW_CLASS, WINDOW_TITLE, 
		 WS_SYSMENU | WS_THICKFRAME| WS_CLIPCHILDREN, CW_USEDEFAULT, 0,
		 CW_USEDEFAULT, 0, hWndParent, NULL, hInst, NULL);

	if (!m_hWnd)
		return FALSE;

	// Set window position
	SetWindowPos(m_hWnd, NULL, prcWindowPosition->left, prcWindowPosition->top,
		prcWindowPosition->right - prcWindowPosition->left, 
		prcWindowPosition->bottom - prcWindowPosition->top,
		SWP_NOZORDER);

	// Create the edit control
	m_hWndEdit = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL, WS_CHILD | WS_VISIBLE | WS_VSCROLL | 
				ES_LEFT | ES_MULTILINE | ES_AUTOVSCROLL | ES_READONLY, 0, 0, 
				100, 100, m_hWnd, (HMENU) IDC_OUTPUT_TEXT, hInst, NULL);

	if (!m_hWndEdit)
		return FALSE;
	
	// Show and update window
	ShowOutputWindow(TRUE);
	UpdateWindow(m_hWnd);

	return TRUE;
}

bool COutputWindow::PrintLine(char szLine[])
{
	// Print a line into the output window

	if (!m_hWnd)
		return FALSE;

	// Set selection start to the end of the edit control
	SendMessage(m_hWndEdit, EM_SETSEL, GetWindowTextLength(m_hWndEdit),
		GetWindowTextLength(m_hWndEdit));

	// Replace the selection with a newline character (if needed)
	if (SendMessage(m_hWndEdit, EM_LINELENGTH, 0, 0) != 0)
		SendMessage(m_hWndEdit, EM_REPLACESEL, (WPARAM) FALSE, (LPARAM) "\r\n");
	
	// Set selection start to the end of the edit control
	SendMessage(m_hWndEdit, EM_SETSEL, GetWindowTextLength(m_hWndEdit),
		GetWindowTextLength(m_hWndEdit));

	// Replace the selection with the passed line
	SendMessage(m_hWndEdit, EM_REPLACESEL, (WPARAM) FALSE, 
		(LPARAM) szLine);

	return TRUE;
}

bool COutputWindow::ShowOutputWindow(bool bVisible)
{
	// Displays / hides the window

	if (!m_hWnd)
		return FALSE;

	if (bVisible)
		ShowWindow(m_hWnd, SW_SHOW);
	else
		ShowWindow(m_hWnd, SW_HIDE);

	return TRUE;
}

bool COutputWindow::IsOutputWindowVisible() 
{
	if (IsWindowVisible(m_hWnd))
		return TRUE; 
	else
		return FALSE;
};

static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;

	switch (message)
	{
		case WM_CREATE:
			return 0;

		case WM_SIZE:
			// Resize the edit control
			SetWindowPos(COutputWindow::GetEditControl(), NULL, 0, 0, 
				LOWORD(lParam), HIWORD(lParam), SWP_NOZORDER);
			break;

		case WM_CLOSE:
			// Don't let the window be closed, just hide it
			ShowWindow(hWnd, SW_HIDE);
			return 0;

		case WM_CTLCOLORSTATIC:
			// Select the correct font for the text box
			hdc = (HDC) wParam;
			SelectObject(hdc, GetStockObject(ANSI_FIXED_FONT));
			return 0;

		case WM_KILLFOCUS:
			FlashWindow(hWnd, TRUE);
			return 0;
	}

	return DefWindowProc (hWnd, message, wParam, lParam);
}

	char gszLine[2];

void COutputWindow::Clear()
{
	gszLine[0] = 0;
	// Set selection start to the end of the edit control
	SendMessage(m_hWndEdit, WM_SETTEXT, 0,
		(LPARAM) gszLine);
}
