/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include <windows.h>
#include "stdafx.h"
#include "PortInfo.h"
#include "Cdebug.h"
extern CDebug Debug;

#define AMOUNT_TO_READ          512
#define NUM_READSTAT_HANDLES    4

extern CDebug Debug;

unsigned char  serial_in_buffer[1024];
int serial_in_head = 0;
int serial_in_tail = 0;

DWORD WINAPI ReaderThread(LPVOID lpV)
{
    OVERLAPPED osReader = {0};  // overlapped structure for read operations
    OVERLAPPED osStatus = {0};  // overlapped structure for status operations
    HANDLE     hArray[NUM_READSTAT_HANDLES];
    DWORD      dwStoredFlags = 0xFFFFFFFF;      // local copy of event flags
    DWORD      dwCommEvent;     // result from WaitCommEvent
    DWORD      dwOvRes;         // result from GetOverlappedResult
    DWORD 	   dwRead;          // bytes actually read
    DWORD      dwRes;           // result from WaitForSingleObject
    BOOL       fWaitingOnRead = FALSE;
    BOOL       fWaitingOnStat = FALSE;
    BOOL       fThreadDone = FALSE;
    char   	   lpBuf[AMOUNT_TO_READ];

	int i;
	char str[128];

    // create two overlapped structures, one for read events
    // and another for status events
    osReader.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
    if (osReader.hEvent == NULL)
        ErrorInComm("CreateEvent (Reader Event)");

    osStatus.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
    if (osStatus.hEvent == NULL)
        ErrorInComm("CreateEvent (Status Event)");

    //   Read events (from ReadFile)
    //   Status events (from WaitCommEvent)
    //   Status message events (from our UpdateStatus)
    //   Thread exit evetns (from shutdown functions)
    hArray[0] = osReader.hEvent;
    hArray[1] = osStatus.hEvent;
    hArray[2] = ghStatusMessageEvent;
    hArray[3] = ghThreadExitEvent;

    while ( !fThreadDone ) {
		Debug.Printf1("Reader Thread");

        if (NOREADING( TTYInfo ))
            fWaitingOnRead = TRUE;
        
        // if no read is outstanding, then issue another one
        if (!fWaitingOnRead) {
            if (!ReadFile(COMDEV(TTYInfo), lpBuf, AMOUNT_TO_READ, &dwRead, &osReader)) {
                if (GetLastError() != ERROR_IO_PENDING)	  // read not delayed?
                    ErrorInComm("ReadFile in ReaderThread");

                fWaitingOnRead = TRUE;
            }
            else {    // read completed immediately
                if ((dwRead != MAX_READ_BUFFER) && SHOWTIMEOUTS(TTYInfo)) {
                    //UpdateStatus("Read timed out immediately.\r\n");
				}

                if (dwRead) {
					for (i = 0; i < dwRead; i++) {
                      sprintf(str,"%02X.",lpBuf[i]);					
                      //OutputABuffer(hTTY, str, 3);
					}
				}
            }
        }

        // If status flags have changed, then reset comm mask.
        // This will cause a pending WaitCommEvent to complete
        if (dwStoredFlags != EVENTFLAGS(TTYInfo)) {
            dwStoredFlags = EVENTFLAGS(TTYInfo);
            if (!SetCommMask(COMDEV(TTYInfo), dwStoredFlags))
                ErrorReporter("SetCommMask");
        }

        // If event checks are not allowed, then make it look
        // like an event check operation is outstanding
        if (NOEVENTS(TTYInfo))
            fWaitingOnStat = TRUE;
        // if no status check is outstanding, then issue another one
        if (!fWaitingOnStat) {
            if (NOEVENTS(TTYInfo))
                fWaitingOnStat = TRUE;
            else {
                if (!WaitCommEvent(COMDEV(TTYInfo), &dwCommEvent, &osStatus)) {
                    if (GetLastError() != ERROR_IO_PENDING)	  // Wait not delayed?
                        ErrorReporter("WaitCommEvent");
                    else
                        fWaitingOnStat = TRUE;
                }
                else {
                    // WaitCommEvent returned immediately
                    //ReportStatusEvent(dwCommEvent);
				}
            }
        }

        // wait for pending operations to complete
        if ( fWaitingOnStat && fWaitingOnRead ) {
            dwRes = WaitForMultipleObjects(NUM_READSTAT_HANDLES, hArray, FALSE, STATUS_CHECK_TIMEOUT);
            switch(dwRes)
            {
                // read completed
                case WAIT_OBJECT_0:
                    if (!GetOverlappedResult(COMDEV(TTYInfo), &osReader, &dwRead, FALSE)) {
                        if (GetLastError() == ERROR_OPERATION_ABORTED) {
                           // UpdateStatus("Read aborted\r\n");
						}
                        else
                            ErrorInComm("GetOverlappedResult (in Reader)");
                    }
                    else {      // read completed successfully
                        if ((dwRead != MAX_READ_BUFFER) && SHOWTIMEOUTS(TTYInfo)) {
                            //UpdateStatus("Read timed out overlapped.\r\n");
						}

                        if (dwRead) {
							for (i = 0; i < dwRead; i++) {
								serial_in_buffer[serial_in_head++] = (unsigned char)lpBuf[i];
								if (serial_in_head == 1024) serial_in_head = 0;
								sprintf(str,"    TR. [%02X]",(unsigned char)lpBuf[i]);
							}
/*							for (i = 0; i < dwRead; i++) {
//							  wdata = (unsigned int)((unsigned char)lpBuf[i]);
							  wdata = (unsigned char)lpBuf[i];
							  data = (unsigned char)lpBuf[i];
							  sprintf(str,"%02X ",(unsigned int)data);
							  printf(str);
							  //OutputABuffer(hTTY, str, 3);
							}
							str[0] = 13;
							str[1] = 10;
							printf("\n");*/
						    //OutputABuffer(hTTY, str, 2);
						}
                    }

                    fWaitingOnRead = FALSE;
                    break;

                // status completed
                case WAIT_OBJECT_0 + 1: 
                    if (!GetOverlappedResult(COMDEV(TTYInfo), &osStatus, &dwOvRes, FALSE)) {
                        if (GetLastError() == ERROR_OPERATION_ABORTED) {
                            //UpdateStatus("WaitCommEvent aborted\r\n");
						}
                        else
                            ErrorInComm("GetOverlappedResult (in Reader)");
                    }
                    else  {     // status check completed successfully
                        //ReportStatusEvent(dwCommEvent);
					}

                    fWaitingOnStat = FALSE;
                    break;

                // status message event
                case WAIT_OBJECT_0 + 2:
                    //StatusMessage();
                    break;

                //
                // thread exit event
                //
                case WAIT_OBJECT_0 + 3:
                    fThreadDone = TRUE;
                    break;

                case WAIT_TIMEOUT:
                    // timeouts are not reported because they happen too often
                    // OutputDebugString("Timeout in Reader & Status checking\n\r");
                    // if status checks are not allowed, then don't issue the
                    // modem status check nor the com stat check
                    if (!NOSTATUS(TTYInfo)) {
                        //CheckModemStatus(FALSE);    // take this opportunity to do
                        //CheckComStat(FALSE);        //   a modem status check and
                                                    //   a comm status check
                    }

                    break;                       

                default:
                    ErrorReporter("WaitForMultipleObjects(Reader & Status handles)");
                    break;
            }
        }
    }

    // close event handles
    CloseHandle(osReader.hEvent);
    CloseHandle(osStatus.hEvent);

    return 1;
}
