/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include <windows.h>
#include <commctrl.h>

#include "stdafx.h"
#include "PortInfo.h"
#include "Cdebug.h"
extern CDebug Debug;

PWRITEREQUEST RemoveFromLinkedList( PWRITEREQUEST );
BOOL QueueForOutput( DWORD, DWORD, char, char *, HANDLE, HWND );
void HandleWriteRequests( void );
void WriterFileStart( DWORD );
void WriterComplete( void );
void WriterAbort( PWRITEREQUEST );
void AddToLinkedList( PWRITEREQUEST );
void SendData(char * lpBuf, DWORD dwToWrite);


DWORD WINAPI WriterThread(LPVOID lpV)
{
    SYSTEM_INFO sysInfo;
    HANDLE hArray[2];
    DWORD dwRes;
    DWORD dwSize;
    BOOL fDone = FALSE;

    // create a heap for WRITE_REQUEST packets
    GetSystemInfo(&sysInfo);
	printf("create a heap for WRITE_REQUEST packets\n");
    ghWriterHeap = HeapCreate(0, sysInfo.dwPageSize*2, sysInfo.dwPageSize*4);
    if (ghWriterHeap == NULL)
        ErrorInComm("HeapCreate (write request heap)");

    // create synchronization events for write requests and file transfers
    ghWriterEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (ghWriterEvent == NULL)
        ErrorInComm("CreateEvent(writ request event)");

    ghTransferCompleteEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (ghTransferCompleteEvent == NULL)
        ErrorInComm("CreateEvent(transfer complete event)");

    // initialize write request linked list
    dwSize = sizeof(WRITEREQUEST);
    gpWriterHead = (struct WRITEREQUEST *)HeapAlloc(ghWriterHeap, HEAP_ZERO_MEMORY, dwSize);
    gpWriterTail = (struct WRITEREQUEST *)HeapAlloc(ghWriterHeap, HEAP_ZERO_MEMORY, dwSize);
    gpWriterHead->pNext = gpWriterTail;
    gpWriterTail->pPrev = gpWriterHead;

    hArray[0] = ghWriterEvent;
    hArray[1] = ghThreadExitEvent;
   
    while ( !fDone ) {
		Debug.Printf1("Writer Thread");
        dwRes = WaitForMultipleObjects(2, hArray, FALSE, WRITE_CHECK_TIMEOUT);
        switch(dwRes)
        {
            case WAIT_TIMEOUT:
		Debug.Printf1("Writer Thread Wait Timeout");
                    break;

            case WAIT_FAILED:
		Debug.Printf1("Writer Thread Wait Failed");
                    ErrorReporter("WaitForMultipleObjects( writer proc )");
                    break;

            // write request event
            case WAIT_OBJECT_0:
		Debug.Printf1("Writer Thread Write Event");
                    HandleWriteRequests();
                    break;
            // thread exit event
            case WAIT_OBJECT_0 + 1:
		Debug.Printf1("Writer Thread Exit");
                    fDone = TRUE;
                    break;
        }
    }

    CloseHandle(ghTransferCompleteEvent);
    CloseHandle(ghWriterEvent);

    // Destroy WRITE_REQUEST heap
    HeapDestroy(ghWriterHeap);
    return 1;
}

void HandleWriteRequests()
{
    PWRITEREQUEST pWrite;
    
    pWrite = gpWriterHead->pNext;
    
    while(pWrite != gpWriterTail) {
        switch(pWrite->dwWriteType)
        {
            case WRITE_CHAR:          SendData(&(pWrite->ch), 1);                break;
            case WRITE_CHAR3:          
								SendData(&(pWrite->ch1), 1);
								SendData(&(pWrite->ch2), 1);
								SendData(&(pWrite->ch3), 1);
								break;
            default:                  ErrorReporter("Bad write request"); 
                                      break;
        }

        //
        // remove current node and get next node
        //
        pWrite = RemoveFromLinkedList(pWrite);
        pWrite = gpWriterHead->pNext;
    }

    return;
}

void WriterComplete()
{
    if (!SetEvent(ghTransferCompleteEvent))
        ErrorReporter("SetEvent (transfer complete event)");

    return;
}

void WriterAbort(PWRITEREQUEST pAbortNode)
{
    PWRITEREQUEST pCurrent;
    PWRITEREQUEST pNextNode;
    BOOL fRes;
    int i = 0;
    char szMessage[30];

    EnterCriticalSection(&gcsWriterHeap);
    // remove all nodes after me
    pCurrent = pAbortNode->pNext;

    while (pCurrent != gpWriterTail) {
        pNextNode = pCurrent->pNext;
        fRes = HeapFree(ghWriterHeap, 0, pCurrent);
        if (!fRes)
            break;
        i++;
        pCurrent = pNextNode;
    }

    pAbortNode->pNext = gpWriterTail;
    gpWriterTail->pPrev = pAbortNode;
    LeaveCriticalSection(&gcsWriterHeap);

    wsprintf(szMessage, "%d packets ignored.\n", i);
    OutputDebugString(szMessage);

    if (!fRes)
        ErrorReporter("HeapFree (Writer heap)");

    if (!SetEvent(ghTransferCompleteEvent))
        ErrorReporter("SetEvent (transfer complete event)");

    return;
}


void SendData(char * lpBuf, DWORD dwToWrite)
{
    OVERLAPPED osWrite = {0};
    HANDLE hArray[2];
    DWORD dwWritten;
    DWORD dwRes;

    if (NOWRITING(TTYInfo))
        return ;

    // create this writes overlapped structure hEvent
    osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
    if (osWrite.hEvent == NULL)
        ErrorInComm("CreateEvent (overlapped write hEvent)");

    hArray[0] = osWrite.hEvent;
    hArray[1] = ghThreadExitEvent;
    
    // issue write
    if (!WriteFile(COMDEV(TTYInfo), lpBuf, dwToWrite, &dwWritten, &osWrite)) {
        if (GetLastError() == ERROR_IO_PENDING) { 
            // write is delayed
            dwRes = WaitForMultipleObjects(2, hArray, FALSE, INFINITE);
            switch(dwRes)
            {
                // write event set
                case WAIT_OBJECT_0:
                            SetLastError(ERROR_SUCCESS);
                            if (!GetOverlappedResult(COMDEV(TTYInfo), &osWrite, &dwWritten, FALSE)) {
                                if (GetLastError() == ERROR_OPERATION_ABORTED) {
                                    //UpdateStatus("Write aborted\r\n");
								}
                                else
                                    ErrorInComm("GetOverlappedResult(in Writer)");
                            }
                            
                            if (dwWritten != dwToWrite) {
                                if ((GetLastError() == ERROR_SUCCESS) && SHOWTIMEOUTS(TTYInfo)) {
                                    //UpdateStatus("Write timed out. (overlapped)\r\n");
								}
                                else
                                    ErrorReporter("Error writing data to port (overlapped)");
                            }
                            break;

                // thread exit event set
                case WAIT_OBJECT_0 + 1:
                            break;

                // wait timed out
                case WAIT_TIMEOUT:
//                            UpdateStatus("Wait Timeout in SendData.\r\n");
                            break;

                case WAIT_FAILED:
                default:    ErrorInComm("WaitForMultipleObjects (WriterGeneric)");
                            break;
            }
        }
        else    
            //
            // writefile failed, but it isn't delayed
            //
            ErrorInComm("WriteFile (in Writer)");
    }
    else {
        //
        // writefile returned immediately
        //
        if (dwWritten != dwToWrite) {
           // UpdateStatus("Write timed out. (immediate)\r\n");
		}
    }

    CloseHandle(osWrite.hEvent);
    
    return;
}

BOOL QueueForOutput(  DWORD dwRequestType, 
                        DWORD dwSize, 
                        char ch, 
                        char * lpBuf, 
                        HANDLE hHeap, 
                        HWND hProgress)
{
    PWRITEREQUEST pWrite;

    //
    // allocate new packet
    //
	if (ghWriterHeap == NULL) printf("Error!: ghWriterHeap == NULL\n");
	//printf("Sizeof(WRITEREQUEST) == %d\n",sizeof(WRITEREQUEST));
    pWrite = (PWRITEREQUEST)HeapAlloc(ghWriterHeap, 0, sizeof(WRITEREQUEST));
    if (pWrite == NULL) {
		printf("HeapAlloc error\n");
        ErrorReporter("HeapAlloc (writer packet)");
        return FALSE;
    }

    // assign packet info
    pWrite->dwWriteType  = dwRequestType;
    pWrite->dwSize       = dwSize;
    pWrite->ch           = ch;
    pWrite->lpBuf        = lpBuf;
    pWrite->hHeap        = hHeap;
    pWrite->hWndProgress = hProgress;

    AddToLinkedList(pWrite);
    
    return TRUE;
}

BOOL QueueForOutput3(  DWORD dwRequestType, 
                        DWORD dwSize, 
                        char ch1,char ch2,char ch3, 
                        char * lpBuf, 
                        HANDLE hHeap, 
                        HWND hProgress)
{
    PWRITEREQUEST pWrite;

    //
    // allocate new packet
    //
	if (ghWriterHeap == NULL) printf("Error!: ghWriterHeap == NULL\n");
	//printf("Sizeof(WRITEREQUEST) == %d\n",sizeof(WRITEREQUEST));
    pWrite = (PWRITEREQUEST)HeapAlloc(ghWriterHeap, 0, sizeof(WRITEREQUEST));
    if (pWrite == NULL) {
		printf("HeapAlloc error\n");
        ErrorReporter("HeapAlloc (writer packet)");
        return FALSE;
    }

    // assign packet info
    pWrite->dwWriteType  = dwRequestType;
    pWrite->dwSize       = dwSize;
    pWrite->ch1          = ch1;
    pWrite->ch2          = ch2;
    pWrite->ch3          = ch3;
    pWrite->lpBuf        = lpBuf;
    pWrite->hHeap        = hHeap;
    pWrite->hWndProgress = hProgress;

    AddToLinkedList(pWrite);
    
    return TRUE;
}

void AddToLinkedList(PWRITEREQUEST pNode)
{
    PWRITEREQUEST pOldLast;
    //
    // add node to linked list
    //
    EnterCriticalSection(&gcsWriterHeap);

    pOldLast = gpWriterTail->pPrev;

    pNode->pNext = gpWriterTail;
    pNode->pPrev = pOldLast;

    pOldLast->pNext = pNode;
    gpWriterTail->pPrev = pNode;

    LeaveCriticalSection(&gcsWriterHeap);

    //
    // notify writer thread that a node has been added
    // 
    if (!SetEvent(ghWriterEvent))
        ErrorReporter("SetEvent( writer packet )");
    
    return;    
}


PWRITEREQUEST RemoveFromLinkedList(PWRITEREQUEST pNode)
{
    PWRITEREQUEST pNextNode;
    PWRITEREQUEST pPrevNode;
    BOOL bRes;

    EnterCriticalSection(&gcsWriterHeap);
    
    pNextNode = pNode->pNext;
    pPrevNode = pNode->pPrev;    
    
    bRes = HeapFree(ghWriterHeap, 0, pNode);
    
    pPrevNode->pNext = pNextNode;
    pNextNode->pPrev = pPrevNode;

    LeaveCriticalSection(&gcsWriterHeap);

    if (!bRes)
        ErrorReporter("HeapFree(write request)");

    return pNextNode;     
}

