/*
  This Evaluation kit software is available free of charge and can be used,
  edited, or reproduced as you see fit. Dallas Semiconductor does not
  assume any liability for the accuracy or correctness of this code.

  It is free to use at your own risk.
  */

#include <windows.h>
#include "stdafx.h"
#include "serial.h"
#include "Cdebug.h"
extern CDebug Debug;

#define PORT_DECLARE_VARS 1
#include "PortInfo.h"
#undef PORT_DECLARE_VARS


int ComErrorCode;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define READFROM_KEY  1
#define READFROM_FILE 2
int datasource = READFROM_KEY;

int initmemoryimage_file(char *filename);
int fileimage_putchar3(unsigned char ch1,unsigned char ch2,unsigned char ch3);
int memimage_putchar3(unsigned char ch1,unsigned char ch2,unsigned char ch3);


extern bool auto_poll;
extern unsigned char memimage[6145];


extern unsigned char  serial_in_buffer[1024];
extern int serial_in_head;
extern int serial_in_tail;

int InitComPort(char *portstr, int baudrate, int dtr) {

// for conveinance sake...
//	initmemoryimage_file("c:\\datadump.hex");

    Debug.Printf1("InitComPort");
    PortInitialize();     
    InitPortInfo();
    if (SetupCommPort(portstr) == NULL) {
		//ComErrorCode = GetLastError();
		return COM_OPEN_ERROR;
	}
    serial_in_head  = serial_in_tail = 0;
//    serial_out_head = serial_out_tail = 0;

	Sleep(2000);
	return 1;
}


static int error_under;

int serial_get_error() {
return error_under;
}

void serial_clear_error() {
	error_under = 0;
}

unsigned char com_getchar() {
    Debug.Printf1("com_getchar");
	unsigned char c;
	if (serial_in_head != serial_in_tail) {
       c = serial_in_buffer[serial_in_tail++];
	   if (serial_in_tail == 1024) {
			serial_in_tail = 0;
	   }
	   return c;
	}
	return 0xFF;
}

int com_in_queued() {
	int head = serial_in_head;
	int tail = serial_in_tail;

	if (tail > head) head+=1024;
	return head - tail;
}

void com_clear_errorcode() {
	ComErrorCode = 0;
}

DWORD com_get_errorcode() {
	return ComErrorCode;
}


int com_putchar(unsigned char ch) {
	if (datasource == READFROM_FILE || !auto_poll) {
		return 1;
	}
    Debug.Printf1("com_putchar");
		QueueForOutput(WRITE_CHAR, 0, (char) ch, NULL, NULL, NULL);
	return 1;
}


int com_putchar3(unsigned char ch1,unsigned char ch2,unsigned char ch3) {
	if (datasource == READFROM_FILE) {
		fileimage_putchar3(ch1,ch2,ch3);
		return 1;
	}
	if (!auto_poll) {
		memimage_putchar3(ch1,ch2,ch3);
		return 1;
	}

    Debug.Printf1("com_putchar3");
		QueueForOutput3(WRITE_CHAR3, 0, (char) ch1,(char) ch2,(char) ch3, NULL, NULL, NULL);
	return 1;
}

int com_incoming() {
  if (serial_in_head != serial_in_tail) return 1;
    else return 0;
}


int wait_for_char() {
  Debug.Printf1("wait_for_char");
  DWORD tick;
  tick = GetTickCount() + 500;
  while (tick > GetTickCount()) {
     if (com_incoming()) return 1;
  }
  return -1;
}



void com_in_clear() {
  serial_in_head = serial_in_tail;
}


int CloseComPort() {
    Debug.Printf1("CloseComPort");
	ShutDownPort();
	return 1;
}




/*
	Read memory image from file code
*/

static unsigned int CRC,CRCb,CRCIN;

static int calc_crc(unsigned int data) {

  unsigned int i;

  for (i=0; i<8; i++) {
    CRCIN = (((CRC^data)&1) << 15);
    data = data >> 1;
    CRC = CRC >> 1;
    if (CRCIN != 0) {
      CRC = CRC ^ (0xA001);
    }
    CRCb = 0xFFFF - CRC;
  }

  return CRCb;
}



unsigned char memoryimage[6145];

// returns -1 if error
int initmemoryimage_file(char *filename) {
	FILE *memoryimagefile;
	int length, pos, count;

	memoryimagefile = fopen(filename,"rb");
	if (memoryimagefile == NULL) return -1;
	// check to see if the length of the file is exactly 6144
	pos = ftell(memoryimagefile);
	fseek(memoryimagefile, 0, SEEK_END);
	// get the length of the file
	length = ftell(memoryimagefile);
	// set the position back to the beginning
	fseek(memoryimagefile, pos, SEEK_SET);
	if (length != 6144) {
		// opps, length of file doesn't match
		fclose(memoryimagefile);
		return -1;
	}

	count = fread(&memoryimage,6144,1,memoryimagefile);
	if (count != 1) {
		// opps, can't read all of the data
		fclose(memoryimagefile);
		return -1;
	}

	fclose(memoryimagefile);
	return 0;
}
	
int fileimage_putchar3(unsigned char ch1,unsigned char ch2,unsigned char ch3) {
	// check if user wants to read page from "device"
	if (ch1 == 0x33) {
		long offset = (long)ch2 * 256L + (long)ch3;
		// point filepointer to beginning of data
		//fseek(memoryimage,offset,SEEK_SET);
		int i;
		unsigned char data;
		CRC = 0;
		for (i = 0; i < 32; i++) {
			data = memoryimage[offset++];
			serial_in_buffer[serial_in_head++] = data;
			if (serial_in_head == 1024) serial_in_head = 0;
			calc_crc(data);
		}
		// insert the "fake" calculated crc into the fifo
		serial_in_buffer[serial_in_head++] = CRCb & 0x00FF;
		if (serial_in_head == 1024) serial_in_head = 0;
		serial_in_buffer[serial_in_head++] = CRCb >> 8;
		if (serial_in_head == 1024) serial_in_head = 0;
	}
	return 1;
}

int memimage_putchar3(unsigned char ch1,unsigned char ch2,unsigned char ch3) {
	// check if user wants to read page from "device"
	if (ch1 == 0x33) {
		long offset = (long)ch2 * 256L + (long)ch3;

		int i;
		unsigned char data;
		CRC = 0;
		for (i = 0; i < 32; i++) {
			data = memimage[offset++];
			serial_in_buffer[serial_in_head++] = data;
			if (serial_in_head == 1024) serial_in_head = 0;
			calc_crc(data);
		}
		// insert the "fake" calculated crc into the fifo
		serial_in_buffer[serial_in_head++] = CRCb & 0x00FF;
		if (serial_in_head == 1024) serial_in_head = 0;
		serial_in_buffer[serial_in_head++] = CRCb >> 8;
		if (serial_in_head == 1024) serial_in_head = 0;
	}
	return 1;
}




